# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

# global variables
app_server_name_1 = nimosclientbase.get_unique_string("appservertc-1")
appserver_to_delete = []

'''SpaceDomainTestCase tests the SpaceDomain object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for SpaceDomain TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for SpaceDomain TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_spacedomains(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().space_domains.list(
        detail=True, limit=2)
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_spacedomains_endrow_beyond(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().space_domains.get(endRow=30)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected. no rows present")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_selectfields_for_spacedomains(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().space_domains.get(
            fields="id,pool_id,pool_name,block_size")
        if resp is not None:
            assert resp.attrs.get("id") is not None
            assert resp.attrs.get("pool_id") is not None
            assert resp.attrs.get("pool_name") is not None
            assert resp.attrs.get("block_size") is not None

    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#


class NimOSClientError(Exception):
    """NimOS Client failed"""


class NimOSAuthenticationError(NimOSClientError):
    """Authentication to NimOS failed"""


class NimOSConnectionError(NimOSClientError):
    """Unable to connect to NimOS"""


class NimOSAPIError(NimOSClientError):
    """NimOS API call failed"""


class NimOSCLIError(NimOSClientError):
    """NimOS CLI command failed"""


class NimOSAPIOperationUnsupported(NimOSClientError):
    """NimOS API Operation not supported"""


class NimOSClientJobTimeoutError(NimOSClientError):
    """NimOS Client job timeout failure"""



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class PerformancePolicy(Resource):
    """Manage performance policies. A performance policy is a set of optimizations including block size, compression, and caching, to ensure that the volume's performance is the best
    configuration for its intended use like databases or log files. By default, a volume uses the \\"default\\" performance policy, which is set to use 4096 byte blocks with full
    compression and caching enabled. For replicated volumes, the same performance policy must exist on each replication partner.

    # Parameters
    id                    : Unique Identifier for the Performance Policy.
    name                  : Name of the Performance Policy.
    full_name             : Fully qualified name of the Performance Policy.
    search_name           : Name of the Performance Policy used for object search.
    description           : Description of a performance policy.
    block_size            : Block Size in bytes to be used by the volumes created with this specific performance policy. Supported block sizes are 4096 bytes (4 KB), 8192 bytes (8
                            KB), 16384 bytes(16 KB), and 32768 bytes (32 KB). Block size of a performance policy cannot be changed once the performance policy is created.
    compress              : Flag denoting if data in the associated volume should be compressed.
    cache                 : Flag denoting if data in the associated volume should be cached.
    cache_policy          : Specifies how data of associated volume should be cached. Supports two policies, 'normal' and 'aggressive'. 'normal' policy caches data but skips in
                            certain conditions such as sequential I/O. 'aggressive' policy will accelerate caching of all data belonging to this volume, regardless of
                            sequentiality.
    space_policy          : Specifies the state of the volume upon space constraint violation such as volume limit violation or volumes above their volume reserve, if the pool
                            free space is exhausted. Supports two policies, 'offline' and 'non_writable'.
    app_category          : Specifies the application category of the associated volume.
    dedupe_enabled        : Specifies if dedupe is enabled for volumes created with this performance policy.
    deprecated            : Specifies if this performance policy is deprecated.
    predefined            : Specifies if this performance policy is predefined (read-only).
    creation_time         : Time when the performance policy was created.
    last_modified         : Time when the performance policy's configurations were last modified.
    sample_rate           : Sample rate value.
    volume_count          : Number of volumes using this performance policy.
    dedupe_override_pools : List of pools that override performance policy's dedupe setting.
    """


class PerformancePolicyList(Collection):
    resource = PerformancePolicy
    resource_type = "performance_policies"



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class NetworkConfig(Resource):
    """Manage group wide network configuration. The three possible network configurations include active, backup and an optional draft configuration.

    # Parameters
    id                                : Identifier for network configuration.
    name                              : Name of the network configuration. Use the name 'draft' when creating a draft configuration.
    mgmt_ip                           : Management IP address for the Group.
    secondary_mgmt_ip                 : Secondary management IP address for the Group.
    role                              : Role of network configuration.
    iscsi_automatic_connection_method : Whether automatic connection method is enabled. Enabling this means means redirecting connections from the specified iSCSI discovery IP
                                        address to the best data IP address based on connection counts.
    iscsi_connection_rebalancing      : Whether rebalancing is enabled. Enabling this means rebalancing iSCSI connections by periodically breaking existing connections that are
                                        out-of-balance, allowing the host to reconnect to a more appropriate data IP address.
    route_list                        : List of static routes.
    subnet_list                       : List of subnet configs.
    array_list                        : List of array network configs.
    group_leader_array                : Name of the group leader array.
    creation_time                     : Time when this net configuration was created.
    last_modified                     : Time when this network configuration was last modified.
    active_since                      : Start time of activity.
    last_active                       : Time of last activity.
    ignore_validation_mask            : Indicates whether to ignore the validation.
    """
    def activate_netconfig(self, ignore_validation_mask, **kwargs):
        """Activate a network configuration.

        # Parameters
        id                     : ID of the netconfig to activate.
        ignore_validation_mask : Whether to ignore validation or not.
        """

        return self._collection.activate_netconfig(
            self.id,
            ignore_validation_mask,
            **kwargs
        )
    def validate_netconfig(self, ignore_validation_mask, **kwargs):
        """Validate a network configuration.

        # Parameters
        id                     : ID of the netconfig to validate.
        ignore_validation_mask : Whether to ignore validation or not.
        """

        return self._collection.validate_netconfig(
            self.id,
            ignore_validation_mask,
            **kwargs
        )


class NetworkConfigList(Collection):
    resource = NetworkConfig
    resource_type = "network_configs"

    def activate_netconfig(self, id, ignore_validation_mask, **kwargs):
        """Activate a network configuration.

        # Parameters
        id                     : ID of the netconfig to activate.
        ignore_validation_mask : Whether to ignore validation or not.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'activate_netconfig',
            id=id,
            ignore_validation_mask=ignore_validation_mask,
            **kwargs
        )

    def validate_netconfig(self, id, ignore_validation_mask, **kwargs):
        """Validate a network configuration.

        # Parameters
        id                     : ID of the netconfig to validate.
        ignore_validation_mask : Whether to ignore validation or not.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'validate_netconfig',
            id=id,
            ignore_validation_mask=ignore_validation_mask,
            **kwargs
        )



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log

# global variables
folders_name_1 = nimosclientbase.get_unique_string("foldertc-1")
folders_to_delete = []

'''FoldersTestCase tests the folders object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Folders TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Folders TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_folders()
    nimosclientbase.log_footer(request.function.__name__)


def delete_folders():
    for folder_id in folders_to_delete:
        nimosclientbase.get_nimos_client().folders.delete(folder_id)
        log(f"Deleted folders with Id '{folder_id}'")
    folders_to_delete.clear()


def create_folders(folder_name, **kwargs):
    resp = nimosclientbase.get_nimos_client().folders.create(
        name=folder_name, **kwargs)
    folder_id = resp.attrs.get("id")
    folders_to_delete.append(folder_id)
    assert resp is not None
    log("Created folder with name "
        f"'{folder_name}' and Id '{folder_id}'")
    assert resp is not None
    return resp


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_folders(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().folders.list(
        detail=True, limit=2)
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_folders(setup_teardown_for_each_test):
    # folder creation requires pool_id. hence first get the ppol id
    pool_resp = nimosclientbase.get_nimos_client().pools.get()
    resp = create_folders(
        folder_name=folders_name_1,
        pool_id=pool_resp.attrs.get("id"),
        description="created by testcase",
        limit_bytes=2000)
    assert resp is not None
    assert resp.attrs.get("name") == folders_name_1
    assert resp.attrs.get("description") == "created by testcase"
    assert resp.attrs.get("limit_bytes") == 2000


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_folders(setup_teardown_for_each_test):
    # folder creation requires pool_id. hence first get the ppol id
    pool_resp = nimosclientbase.get_nimos_client().pools.get()
    resp = create_folders(
        folder_name=folders_name_1,
        pool_id=pool_resp.attrs.get("id"),
        description="created by testcase",
        limit_bytes=2000)
    assert resp is not None
    assert resp.attrs.get("name") == folders_name_1
    assert resp.attrs.get("description") == "created by testcase"
    assert resp.attrs.get("limit_bytes") == 2000

    # update few fields
    update_resp = nimosclientbase.get_nimos_client().folders.update(
        id=resp.attrs.get("id"),
        description="modified by testcase",
        limit_bytes=4000,
        name="folderupdatetestcase")
    update_resp is not None
    assert update_resp.attrs.get("name") == "folderupdatetestcase"
    assert update_resp.attrs.get("description") == "modified by testcase"
    assert update_resp.attrs.get("limit_bytes") == 4000



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#

from .restclient import NimOSAPIClient

from .api.access_control_records import AccessControlRecordList
from .api.active_directory_memberships import ActiveDirectoryMembershipList
from .api.alarms import AlarmList
from .api.application_categories import ApplicationCategoryList
from .api.application_servers import ApplicationServerList
from .api.arrays import ArrayList
from .api.audit_log import AuditLogList
from .api.chap_users import ChapUserList
from .api.controllers import ControllerList
from .api.disks import DiskList
from .api.events import EventList
from .api.fibre_channel_configs import FibreChannelConfigList
from .api.fibre_channel_initiator_aliases import FibreChannelInitiatorAliasList
from .api.fibre_channel_interfaces import FibreChannelInterfaceList
from .api.fibre_channel_ports import FibreChannelPortList
from .api.fibre_channel_sessions import FibreChannelSessionList
from .api.folders import FolderList
from .api.groups import GroupList
from .api.initiator_groups import InitiatorGroupList
from .api.initiators import InitiatorList
from .api.jobs import JobList
from .api.key_managers import KeyManagerList
from .api.master_key import MasterKeyList
from .api.network_configs import NetworkConfigList
from .api.network_interfaces import NetworkInterfaceList
from .api.performance_policies import PerformancePolicyList
from .api.pools import PoolList
from .api.protection_schedules import ProtectionScheduleList
from .api.protection_templates import ProtectionTemplateList
from .api.protocol_endpoints import ProtocolEndpointList
from .api.replication_partners import ReplicationPartnerList
from .api.shelves import ShelfList
from .api.snapshot_collections import SnapshotCollectionList
from .api.snapshots import SnapshotList
from .api.software_versions import SoftwareVersionList
from .api.space_domains import SpaceDomainList
from .api.subnets import SubnetList
from .api.tokens import TokenList
from .api.user_groups import UserGroupList
from .api.user_policies import UserPolicyList
from .api.users import UserList
from .api.versions import VersionList
from .api.volume_collections import VolumeCollectionList
from .api.volumes import VolumeList
from .api.witnesses import WitnessList


class NimOSClient:
    """Nimble client for NimOS v1 REST API."""

    def __init__(self, hostname, username, password, job_timeout=60, port=5392):
        self._client = NimOSAPIClient(hostname, username, password, job_timeout, port)

    @property
    def access_control_records(self):
        return AccessControlRecordList(self._client)

    @property
    def active_directory_memberships(self):
        return ActiveDirectoryMembershipList(self._client)

    @property
    def alarms(self):
        return AlarmList(self._client)

    @property
    def application_categories(self):
        return ApplicationCategoryList(self._client)

    @property
    def application_servers(self):
        return ApplicationServerList(self._client)

    @property
    def arrays(self):
        return ArrayList(self._client)

    @property
    def audit_log(self):
        return AuditLogList(self._client)

    @property
    def chap_users(self):
        return ChapUserList(self._client)

    @property
    def controllers(self):
        return ControllerList(self._client)

    @property
    def disks(self):
        return DiskList(self._client)

    @property
    def events(self):
        return EventList(self._client)

    @property
    def fibre_channel_configs(self):
        return FibreChannelConfigList(self._client)

    @property
    def fibre_channel_initiator_aliases(self):
        return FibreChannelInitiatorAliasList(self._client)

    @property
    def fibre_channel_interfaces(self):
        return FibreChannelInterfaceList(self._client)

    @property
    def fibre_channel_ports(self):
        return FibreChannelPortList(self._client)

    @property
    def fibre_channel_sessions(self):
        return FibreChannelSessionList(self._client)

    @property
    def folders(self):
        return FolderList(self._client)

    @property
    def groups(self):
        return GroupList(self._client)

    @property
    def initiator_groups(self):
        return InitiatorGroupList(self._client)

    @property
    def initiators(self):
        return InitiatorList(self._client)

    @property
    def jobs(self):
        return JobList(self._client)

    @property
    def key_managers(self):
        return KeyManagerList(self._client)

    @property
    def master_key(self):
        return MasterKeyList(self._client)

    @property
    def network_configs(self):
        return NetworkConfigList(self._client)

    @property
    def network_interfaces(self):
        return NetworkInterfaceList(self._client)

    @property
    def performance_policies(self):
        return PerformancePolicyList(self._client)

    @property
    def pools(self):
        return PoolList(self._client)

    @property
    def protection_schedules(self):
        return ProtectionScheduleList(self._client)

    @property
    def protection_templates(self):
        return ProtectionTemplateList(self._client)

    @property
    def protocol_endpoints(self):
        return ProtocolEndpointList(self._client)

    @property
    def replication_partners(self):
        return ReplicationPartnerList(self._client)

    @property
    def shelves(self):
        return ShelfList(self._client)

    @property
    def snapshot_collections(self):
        return SnapshotCollectionList(self._client)

    @property
    def snapshots(self):
        return SnapshotList(self._client)

    @property
    def software_versions(self):
        return SoftwareVersionList(self._client)

    @property
    def space_domains(self):
        return SpaceDomainList(self._client)

    @property
    def subnets(self):
        return SubnetList(self._client)

    @property
    def tokens(self):
        return TokenList(self._client)

    @property
    def user_groups(self):
        return UserGroupList(self._client)

    @property
    def user_policies(self):
        return UserPolicyList(self._client)

    @property
    def users(self):
        return UserList(self._client)

    @property
    def versions(self):
        return VersionList(self._client)

    @property
    def volume_collections(self):
        return VolumeCollectionList(self._client)

    @property
    def volumes(self):
        return VolumeList(self._client)

    @property
    def witnesses(self):
        return WitnessList(self._client)



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Group(Resource):
    """Group is a collection of arrays operating together organized into storage pools.

    # Parameters
    id                                     : Identifier of the group.
    name                                   : Name of the group.
    smtp_server                            : Hostname or IP Address of SMTP Server.
    smtp_port                              : Port number of SMTP Server.
    smtp_auth_enabled                      : Whether SMTP Server requires authentication.
    smtp_auth_username                     : Username to authenticate with SMTP Server.
    smtp_auth_password                     : Password to authenticate with SMTP Server.
    smtp_encrypt_type                      : Level of encryption for SMTP. Requires use of SMTP Authentication if encryption is enabled.
    autosupport_enabled                    : Whether to send autosupport.
    allow_analytics_gui                    : Specify whether to allow HPE Nimble Storage to use Google Analytics in the GUI.  HPE Nimble Storage uses Google Analytics to gather
                                             data related to GUI usage.  The data gathered is used to evaluate and improve the product.
    allow_support_tunnel                   : Whether to allow support tunnel.
    proxy_server                           : Hostname or IP Address of HTTP Proxy Server. Setting this attribute to an empty string will unset all proxy settings.
    proxy_port                             : Proxy Port of HTTP Proxy Server.
    proxy_username                         : Username to authenticate with HTTP Proxy Server.
    proxy_password                         : Password to authenticate with HTTP Proxy Server.
    alert_to_email_addrs                   : Comma-separated list of email addresss to receive emails.
    send_alert_to_support                  : Whether to send alert to Support.
    alert_from_email_addr                  : From email address to use while sending emails.
    alert_min_level                        : Minimum level of alert to be notified.
    isns_enabled                           : Whether iSNS is enabled.
    isns_server                            : Hostname or IP Address of iSNS Server.
    isns_port                              : Port number for iSNS Server.
    snmp_trap_enabled                      : Whether to enable SNMP traps.
    snmp_trap_host                         : Hostname or IP Address to send SNMP traps.
    snmp_trap_port                         : Port number of SNMP trap host.
    snmp_get_enabled                       : Whether to accept SNMP get commands.
    snmp_community                         : Community string to be used with SNMP.
    snmp_get_port                          : Port number to which SNMP get requests should be sent.
    snmp_sys_contact                       : Name of the SNMP administrator.
    snmp_sys_location                      : Location of the group.
    domain_name                            : Domain name for this group.
    dns_servers                            : IP addresses for this group's dns servers.
    ntp_server                             : Either IP address or hostname of the NTP server for this group.
    timezone                               : Timezone in which this group is located.
    user_inactivity_timeout                : The amount of time in seconds that the user session is inactive before timing out.
    syslogd_enabled                        : Is syslogd enabled on this system.
    syslogd_server                         : Hostname of the syslogd server.
    syslogd_port                           : Port number for syslogd server.
    syslogd_servers                        : Hostname and/or port of the syslogd servers.
    vvol_enabled                           : Are vvols enabled on this group.
    iscsi_enabled                          : Whether iSCSI is enabled on this group.
    fc_enabled                             : Whether FC is enabled on this group.
    unique_name_enabled                    : Are new volume and volume collection names transformed on this group.
    access_protocol_list                   : Protocol used to access this group.
    group_target_enabled                   : Is group_target enabled on this group.
    default_iscsi_target_scope             : Newly created volumes are exported under iSCSI Group Target or iSCSI Volume Target.
    tdz_enabled                            : Is Target Driven Zoning (TDZ) enabled on this group.
    tdz_prefix                             : Target Driven Zoning (TDZ) prefix for peer zones created by TDZ.
    group_target_name                      : Iscsi target name for this group.
    default_volume_reserve                 : Amount of space to reserve for a volume as a percentage of volume size.
    default_volume_warn_level              : Default threshold for volume space usage as a percentage of volume size above which an alert is raised.
    default_volume_limit                   : Default limit for a volume space usage as a percentage of volume size. Volume will be taken offline/made non-writable on exceeding its
                                             limit.
    default_snap_reserve                   : Amount of space to reserve for snapshots of a volume as a percentage of volume size.
    default_snap_warn_level                : Default threshold for snapshot space usage of a volume as a percentage of volume size above which an alert is raised.
    default_snap_limit                     : This attribute is deprecated. The array does not limit a volume's snapshot space usage. The attribute is ignored on input and returns
                                             max int64 value on output.
    default_snap_limit_percent             : This attribute is deprecated. The array does not limit a volume's snapshot space usage. The attribute is ignored on input and returns
                                             -1 on output.
    alarms_enabled                         : Whether alarm feature is enabled.
    vss_validation_timeout                 : The amount of time in seconds to validate Microsoft VSS application synchronization before timing out.
    auto_switchover_enabled                : Whether automatic switchover of Group management services feature is enabled.
    auto_switchover_messages               : List of validation messages for automatic switchover of Group Management. This will be empty when there are no conflicts found.
    merge_state                            : State of group merge.
    merge_group_name                       : Group that we're being merged with.
    tlsv1_enabled                          : Enable or disable TLSv1.0 and TLSv1.1.
    cc_mode_enabled                        : Enable or disable Common Criteria mode.
    group_snapshot_ttl                     : Snapshot Time-to-live(TTL) configured at group level for automatic deletion of unmanaged snapshots. Value 0 indicates unlimited TTL.
    autoclean_unmanaged_snapshots_ttl_unit : Unit for unmanaged snapshot time to live.
    autoclean_unmanaged_snapshots_enabled  : Whether autoclean unmanaged snapshots feature is enabled.
    leader_array_name                      : Name of the array where the group Management Service is running.
    leader_array_serial                    : Serial number of the array where the group Management Service is running.
    management_service_backup_array_name   : Name of the array where backup the group Management Service is running.
    management_service_backup_status       : HA status of the group Management Service.
    failover_mode                          : Failover mode of the group Management Service.
    witness_status                         : Witness status from group Management Service array and group Management Service backup array.
    member_list                            : Members of this group.
    compressed_vol_usage_bytes             : Compressed usage of volumes in the group.
    compressed_snap_usage_bytes            : Compressed usage of snapshots in the group.
    uncompressed_vol_usage_bytes           : Uncompressed usage of volumes in the group.
    uncompressed_snap_usage_bytes          : Uncompressed usage of snapshots in the group.
    usable_capacity_bytes                  : Usable capacity bytes of the group.
    usage                                  : Used space of the group in bytes.
    raw_capacity                           : Total capacity of the group.
    usable_cache_capacity                  : Usable cache capacity of the group.
    raw_cache_capacity                     : Total cache capacity of the group.
    snap_usage_populated                   : Total snapshot usage as if each snapshot is deep copy of the volume.
    pending_deletes                        : Usage for blocks that are not yet deleted.
    num_connections                        : Number of connections to the group.
    vol_compression_ratio                  : Compression ratio of volumes in the group.
    snap_compression_ratio                 : Compression ratio of snapshots in the group.
    compression_ratio                      : Compression savings for the group expressed as ratio.
    dedupe_ratio                           : Dedupe savings for the group expressed as ratio.
    clone_ratio                            : Clone savings for the group expressed as ratio.
    vol_thin_provisioning_ratio            : Thin provisioning savings for volumes in the group expressed as ratio.
    savings_ratio                          : Overall savings in the group expressed as ratio.
    data_reduction_ratio                   : Space savings in the group that does not include thin-provisioning savings expressed as ratio.
    savings_dedupe                         : Space usage savings in the group due to deduplication.
    savings_compression                    : Space usage savings in the group due to compression.
    savings_clone                          : Space usage savings in the group due to cloning of volumes.
    savings_vol_thin_provisioning          : Space usage savings in the group due to thin provisioning of volumes.
    savings_data_reduction                 : Space usage savings in the group that does not include thin-provisioning savings.
    savings                                : Overall space usage savings in the group.
    free_space                             : Free space of the pool in bytes.
    unused_reserve_bytes                   : Reserved space that is not utilized.
    usage_valid                            : Indicates whether the usage of group is valid.
    space_info_valid                       : Is space info for this group valid.
    version_current                        : Version of software running on the group.
    version_target                         : Desired software version for the group.
    version_rollback                       : Rollback software version for the group.
    update_state                           : Group update state.
    update_start_time                      : Start time of last update.
    update_end_time                        : End time of last update.
    update_array_names                     : Arrays in the group undergoing update.
    update_progress_msg                    : Group update detailed progress message.
    update_error_code                      : If the software update has failed, this indicates the error code corresponding to the failure.
    update_downloading                     : Is software update package currently downloading.
    update_download_error_code             : If the software download has failed, this indicates the error code corresponding to the failure.
    update_download_start_time             : Start time of last update.
    update_download_end_time               : End time of last update.
    iscsi_automatic_connection_method      : Is iscsi reconnection automatic.
    iscsi_connection_rebalancing           : Does iscsi automatically rebalance connections.
    repl_throttled_bandwidth               : Current bandwidth throttle for replication, expressed either as megabits per second or as -1 to indicate that there is no throttle.
    repl_throttled_bandwidth_kbps          : Current bandwidth throttle for replication, expressed either as kilobits per second or as -1 to indicate that there is no throttle.
    repl_throttle_list                     : All the replication bandwidth limits on the system.
    volume_migration_status                : Status of data migration activity related to volumes being relocated to different pools.
    array_unassign_migration_status        : Data migration status for arrays being removed from their pool.
    data_rebalance_status                  : Status of data rebalancing operations for pools in the group.
    scsi_vendor_id                         : SCSI vendor ID.
    encryption_config                      : How encryption is configured for this group.
    last_login                             : Time and user of last login to this group.
    num_snaps                              : Number of snapshots in the group.
    num_snapcolls                          : Number of snapshot collections in this group.
    date                                   : Unix epoch time local to the group.
    login_banner_message                   : The message for the login banner that is displayed during user login activity.
    login_banner_after_auth                : Should the banner be displayed before the user credentials are prompted or after prompting the user credentials.
    login_banner_reset                     : This will reset the banner to the version of the installed NOS. When login_banner_after_auth is specified, login_banner_reset can not
                                             be set to true.
    snap_retn_meter_high                   : Threshold for considering a volume as high retention.
    snap_retn_meter_very_high              : Threshold for considering a volume as very high retention.
    """
    def reboot(self, **kwargs):
        """Reboot all arrays in the group.

        # Parameters
        id : ID of the group to reboot.
        job_timeout: Job timeout in seconds.
        """

        return self._collection.reboot(
            self.id,
            **kwargs
        )
    def halt(self, **kwargs):
        """Halt all arrays in the group.

        # Parameters
        id    : ID of the group to halt.
        force : Halt remaining arrays when one or more is unreachable.
        job_timeout: Job timeout in seconds.
        """

        return self._collection.halt(
            self.id,
            **kwargs
        )
    def test_alert(self, level, **kwargs):
        """Generate a test alert.

        # Parameters
        id    : ID of the group.
        level : Level of the test alert.
        """

        return self._collection.test_alert(
            self.id,
            level,
            **kwargs
        )
    def software_update_precheck(self, **kwargs):
        """Run software update precheck.

        # Parameters
        id                 : ID of the group.
        skip_precheck_mask : Flag to allow skipping certain types of prechecks.
        """

        return self._collection.software_update_precheck(
            self.id,
            **kwargs
        )
    def software_update_start(self, **kwargs):
        """Update the group software to the downloaded version.

        # Parameters
        id                    : ID of the group.
        skip_start_check_mask : Flag to allow skipping certain types of checks.
        """

        return self._collection.software_update_start(
            self.id,
            **kwargs
        )
    def software_download(self, version, **kwargs):
        """Download software update package.

        # Parameters
        id      : ID of the group.
        version : Version string to download.
        force   : Flag to force download.
        """

        return self._collection.software_download(
            self.id,
            version,
            **kwargs
        )
    def software_cancel_download(self, **kwargs):
        """Cancel ongoing download of software.

        # Parameters
        id : ID of the group.
        """

        return self._collection.software_cancel_download(
            self.id,
            **kwargs
        )
    def software_update_resume(self, **kwargs):
        """Resume stopped software update.

        # Parameters
        id : ID of the group.
        """

        return self._collection.software_update_resume(
            self.id,
            **kwargs
        )
    def get_group_discovered_list(self, **kwargs):
        """Get list of discovered groups with arrays that are initialized.

        # Parameters
        id         : ID of the group.
        group_name : Name of the group requested to be discovered.
        """

        return self._collection.get_group_discovered_list(
            self.id,
            **kwargs
        )
    def validate_merge(self, src_group_ip, src_group_name, src_password, src_username, **kwargs):
        """Perform group merge validation.

        # Parameters
        id                     : ID of the group.
        src_group_name         : Name of the source group.
        src_group_ip           : IP address of the source group.
        src_username           : Username of the source group.
        src_password           : Password of the source group.
        src_passphrase         : Source group encryption passphrase.
        skip_secondary_mgmt_ip : Skip check for secondary management IP address.
        """

        return self._collection.validate_merge(
            self.id,
            src_group_ip,
            src_group_name,
            src_password,
            src_username,
            **kwargs
        )
    def merge(self, src_group_ip, src_group_name, src_password, src_username, **kwargs):
        """Perform group merge with the specified group.

        # Parameters
        id                     : ID of the group.
        src_group_name         : Name of the source group.
        src_group_ip           : IP address of the source group.
        src_username           : Username of the source group.
        src_password           : Password of the source group.
        src_passphrase         : Source group encryption passphrase.
        force                  : Ignore warnings and forcibly merge specified group with this group.
        skip_secondary_mgmt_ip : Skip check for secondary management IP address.
        job_timeout            : Job timeout in seconds.
        """

        return self._collection.merge(
            self.id,
            src_group_ip,
            src_group_name,
            src_password,
            src_username,
            **kwargs
        )
    def get_eula(self, **kwargs):
        """Get URL to download EULA contents.

        # Parameters
        id     : ID of the group.
        locale : Locale of EULA contents. Default is en.
        format : Format of EULA contents. Default is HTML.
        phase  : Phase of EULA contents. Default is setup.
        force  : Flag to force EULA.
        """

        return self._collection.get_eula(
            self.id,
            **kwargs
        )
    def check_migrate(self, **kwargs):
        """Check if the group Management Service can be migrated to the group Management Service backup array.

        # Parameters
        id : ID of the group.
        """

        return self._collection.check_migrate(
            self.id,
            **kwargs
        )
    def migrate(self, **kwargs):
        """Migrate the group Management Service to the current group Management Service backup array.

        # Parameters
        id : ID of the group.
        """

        return self._collection.migrate(
            self.id,
            **kwargs
        )
    def get_timezone_list(self, **kwargs):
        """Get list of group timezones.

        # Parameters
        id : ID of the group.
        """

        return self._collection.get_timezone_list(
            self.id,
            **kwargs
        )

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")


class GroupList(Collection):
    resource = Group
    resource_type = "groups"

    def reboot(self, id, **kwargs):
        """Reboot all arrays in the group.

        # Parameters
        id : ID of the group to reboot.
        job_timeout: Job timeout in seconds.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'reboot',
            id=id,
            **kwargs
        )

    def halt(self, id, **kwargs):
        """Halt all arrays in the group.

        # Parameters
        id    : ID of the group to halt.
        force : Halt remaining arrays when one or more is unreachable.
        job_timeout: Job timeout in seconds.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'halt',
            id=id,
            **kwargs
        )

    def test_alert(self, id, level, **kwargs):
        """Generate a test alert.

        # Parameters
        id    : ID of the group.
        level : Level of the test alert.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'test_alert',
            id=id,
            level=level,
            **kwargs
        )

    def software_update_precheck(self, id, **kwargs):
        """Run software update precheck.

        # Parameters
        id                 : ID of the group.
        skip_precheck_mask : Flag to allow skipping certain types of prechecks.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'software_update_precheck',
            id=id,
            **kwargs
        )

    def software_update_start(self, id, **kwargs):
        """Update the group software to the downloaded version.

        # Parameters
        id                    : ID of the group.
        skip_start_check_mask : Flag to allow skipping certain types of checks.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'software_update_start',
            id=id,
            **kwargs
        )

    def software_download(self, id, version, **kwargs):
        """Download software update package.

        # Parameters
        id      : ID of the group.
        version : Version string to download.
        force   : Flag to force download.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'software_download',
            id=id,
            version=version,
            **kwargs
        )

    def software_cancel_download(self, id, **kwargs):
        """Cancel ongoing download of software.

        # Parameters
        id : ID of the group.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'software_cancel_download',
            id=id,
            **kwargs
        )

    def software_update_resume(self, id, **kwargs):
        """Resume stopped software update.

        # Parameters
        id : ID of the group.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'software_update_resume',
            id=id,
            **kwargs
        )

    def get_group_discovered_list(self, id, **kwargs):
        """Get list of discovered groups with arrays that are initialized.

        # Parameters
        id         : ID of the group.
        group_name : Name of the group requested to be discovered.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'get_group_discovered_list',
            id=id,
            **kwargs
        )

    def validate_merge(self, id, src_group_ip, src_group_name, src_password, src_username, **kwargs):
        """Perform group merge validation.

        # Parameters
        id                     : ID of the group.
        src_group_name         : Name of the source group.
        src_group_ip           : IP address of the source group.
        src_username           : Username of the source group.
        src_password           : Password of the source group.
        src_passphrase         : Source group encryption passphrase.
        skip_secondary_mgmt_ip : Skip check for secondary management IP address.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'validate_merge',
            id=id,
            src_group_ip=src_group_ip,
            src_group_name=src_group_name,
            src_password=src_password,
            src_username=src_username,
            **kwargs
        )

    def merge(self, id, src_group_ip, src_group_name, src_password, src_username, **kwargs):
        """Perform group merge with the specified group.

        # Parameters
        id                     : ID of the group.
        src_group_name         : Name of the source group.
        src_group_ip           : IP address of the source group.
        src_username           : Username of the source group.
        src_password           : Password of the source group.
        src_passphrase         : Source group encryption passphrase.
        force                  : Ignore warnings and forcibly merge specified group with this group.
        skip_secondary_mgmt_ip : Skip check for secondary management IP address.
        job_timeout            : Job timeout in seconds.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'merge',
            id=id,
            src_group_ip=src_group_ip,
            src_group_name=src_group_name,
            src_password=src_password,
            src_username=src_username,
            **kwargs
        )

    def get_eula(self, id, **kwargs):
        """Get URL to download EULA contents.

        # Parameters
        id     : ID of the group.
        locale : Locale of EULA contents. Default is en.
        format : Format of EULA contents. Default is HTML.
        phase  : Phase of EULA contents. Default is setup.
        force  : Flag to force EULA.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'get_eula',
            id=id,
            **kwargs
        )

    def check_migrate(self, id, **kwargs):
        """Check if the group Management Service can be migrated to the group Management Service backup array.

        # Parameters
        id : ID of the group.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'check_migrate',
            id=id,
            **kwargs
        )

    def migrate(self, id, **kwargs):
        """Migrate the group Management Service to the current group Management Service backup array.

        # Parameters
        id : ID of the group.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'migrate',
            id=id,
            **kwargs
        )

    def get_timezone_list(self, id, **kwargs):
        """Get list of group timezones.

        # Parameters
        id : ID of the group.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'get_timezone_list',
            id=id,
            **kwargs
        )

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Version(Resource):
    """Show the API version.

    # Parameters
    name             : API version number.
    software_version : Software version of the group.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class VersionList(Collection):
    resource = Version
    resource_type = "versions"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class MasterKey(Resource):
    """Manage the master key. Data encryption keys for volumes are encrypted by using a master key that must be initialized before encrypted volumes can be created. The master key in
    turn is protected by a passphrase that is set when the master key is created. The passphrase may have to be entered to enable the master key when it is not available, for
    example, after an array reboot.

    # Parameters
    id             : Identifier of the master key.
    name           : Name of the master key. The only allowed value is "default".
    passphrase     : Passphrase used to protect the master key, required during creation, enabling/disabling the key and change the passphrase to a new value.
    new_passphrase : When changing the passphrase, this attribute specifies the new value of the passphrase.
    active         : Whether the master key is active or not.
    purge_age      : Default minimum age (in hours) of inactive encryption keys to be purged. '0' indicates to purge keys immediately.
    """
    def purge_inactive(self, **kwargs):
        """Purges encryption keys that have been inactive for the age or longer. If you do not specify an age, the keys will be purged immediately.

        # Parameters
        id  : Identifier for the master key.
        age : Minimum age (in hours) of inactive encryption keys to be purged. '0' indicates to purge the keys immediately.
        """

        return self._collection.purge_inactive(
            self.id,
            **kwargs
        )


class MasterKeyList(Collection):
    resource = MasterKey
    resource_type = "master_key"

    def purge_inactive(self, id, **kwargs):
        """Purges encryption keys that have been inactive for the age or longer. If you do not specify an age, the keys will be purged immediately.

        # Parameters
        id  : Identifier for the master key.
        age : Minimum age (in hours) of inactive encryption keys to be purged. '0' indicates to purge the keys immediately.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'purge_inactive',
            id=id,
            **kwargs
        )



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST,array_version, log_to_file as log
from nimbleclient import exceptions


'''Controller TestCase tests the controller object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Controller TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Controller TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_controllers(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().controllers.get()
    assert resp is not None


# endRow param is supported with array_versions > 5.2
@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_controllers_endrow_beyond(setup_teardown_for_each_test):

    if nimosclientbase.is_array_version_above_or_equal("5.2"):
        try:
            resp = nimosclientbase.get_nimos_client().controllers.list(
                detail=True, endRow=30)
            assert resp is not None
        except exceptions.NimOSAPIError as ex:
            if "SM_end_row_beyond_total_rows" in str(ex):
                log("Failed as expected.no rows present")
            else:
                log(f"Failed with exception message : {str(ex)}")
                raise ex
    else:
        log(f"Skipped this testcase as it is not supported for array version {array_version} ")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_select_fields_for_controllers(setup_teardown_for_each_test):
    if nimosclientbase.is_array_version_above_or_equal("5.2"):
        try:
            resp = nimosclientbase.get_nimos_client().controllers.get(
                fields="name,hostname")
            assert resp is not None
            assert resp.attrs.get("name") is not None
            assert resp.attrs.get("hostname") is not None
            # try asserting a value which was not querying
            assert resp.attrs.get("port") is None
            assert resp.attrs.get("creation_time") is None
        except exceptions.NimOSAPIError as ex:
            if "SM_end_row_beyond_total_rows" in str(ex):
                log("Failed as expected")
            else:
                log(f"Failed with exception message : {str(ex)}")
                raise ex
    else:
        log(f"Skipped this testcase as it is not supported for array version {array_version} ")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

user_name1 = "TestCaseUser"
user_to_delete = []

'''ClientUserTestCase tests the user object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for ClienttUser TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for ClientUser TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_user()
    nimosclientbase.log_footer(request.function.__name__)


def delete_user():
    for user_id in user_to_delete:
        try:
            resp = nimosclientbase.get_nimos_client().users.delete(id=user_id)
            assert resp is not None
            log(f" Deleted user with id '{user_id}'")
        except exceptions.NimOSAPIError as ex:
            log(f"Failed with exception message : {str(ex)}")
            raise ex
    user_to_delete.clear()


def create_user(user_name1, **kwargs):
    resp = nimosclientbase.get_nimos_client().users.create(
        name=user_name1, **kwargs)
    user_id = resp.attrs.get("id")
    user_to_delete.append(user_id)
    assert resp is not None
    log(f"Creating testuser with name '{user_name1}' and ID '{user_id}'")
    return resp


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_user_name_with_unsupported_char(setup_teardown_for_each_test):
    # invalid char present in user_name1.
    try:
        create_user(user_name1, password="password-91")
    except exceptions.NimOSAPIError:
        log(f"Failed as expected. Invalid user_name : {user_name1}")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_user_name_with_more_than_allowed_characters(
        setup_teardown_for_each_test):
    # only 32 char is allowed for user_name1.
    try:
        username = user_name1+user_name1+user_name1+user_name1 + \
            user_name1+user_name1+user_name1+user_name1
        create_user(username, password="password-91")
    except exceptions.NimOSAPIError:
        log(f"Failed as expected. invalid user_name : {username}")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_user_name_with_valid_details(setup_teardown_for_each_test):
    try:
        resp = create_user(user_name1, password="password-91",
                           full_name="alok ranjan",
                           role="administrator",
                           disabled=True)

        # check the role
        assert resp.attrs.get("role") == "administrator"
        # change the role to guest
        resp = nimosclientbase.get_nimos_client().users.update(
            resp.attrs.get("id"), role="guest")
        assert resp.attrs.get("role") == "guest"
    except exceptions.NimOSAPIError:
        log(f"Failed as expected. Invalid user_name : {user_name1}")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_user_password_length(setup_teardown_for_each_test):
    try:
        password = "pass91*"  # minimum length should be 8 excluding & and [];'
        create_user(user_name1, password=password,
                    full_name="alok ranjan",
                    role="administrator",
                    disabled=True)
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. password length is short : {password}")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_user_password_invalid_character(setup_teardown_for_each_test):
    try:
        password = "pass91*asda&"
        create_user(user_name1, password=password,
                    full_name="alok ranjan",
                    role="administrator",
                    disabled=True)
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log("Failed as expected. "
                f"Invalid character in password : '{password}'")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class ProtocolEndpoint(Resource):
    """Protocol endpoints are administrative logical units (LUs) in an LU conglomerate to be used with VMware Virtual Volumes.

    # Parameters
    id                     : Identifier for the protocol endpoint.
    name                   : Name of the protocol endpoint.
    description            : Text description of the protocol endpoint.
    pool_name              : Name of the pool where the protocol endpoint resides. If pool option is not specified, protocol endpoint is assigned to the default pool.
    pool_id                : Identifier associated with the pool in the storage pool table.
    state                  : Operational state of protocol endpoint.
    serial_number          : Identifier associated with the protocol endpoint for the SCSI protocol.
    target_name            : The iSCSI Qualified Name (IQN) or the Fibre Channel World Wide Node Name (WWNN) of the target protocol endpoint.
    group_specific_ids     : External UID is used to compute the serial number and IQN which never change even if the running group changes (e.g. after group merge).
                             Group-specific IDs determine whether external UID is used for computing serial number and IQN.
    creation_time          : Time when this protocol endpoint was created.
    last_modified          : Time when this protocol endpoint was last modified.
    num_connections        : Number of connections via this protocol endpoint.
    num_iscsi_connections  : Number of iSCSI connections via this protocol endpoint.
    num_fc_connections     : Number of FC connections via this protocol endpoint.
    access_control_records : List of access control records that apply to this protocol endpoint.
    iscsi_sessions         : List of iSCSI sessions connected to this protocol endpoint.
    fc_sessions            : List of FC sessions connected to this protocol endpoint.
    access_protocol        : Access protocol of the protocol endpoint. Only initiator groups with the same access protocol can access the protocol endpoint. If not specified in
                             the creation request, it will be the access protocol supported by the group. If the group supports multiple protocols, the default will be Fibre
                             Channel.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class ProtocolEndpointList(Collection):
    resource = ProtocolEndpoint
    resource_type = "protocol_endpoints"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class ApplicationCategory(Resource):
    """Provides the list of application categories that are available, to classify volumes depending on the applications that use them.

    # Parameters
    id             : Identifier for the application category.
    name           : Name of application category.
    dedupe_enabled : Specifies if dedupe is enabled for performance policies associated with this application category.
    creation_time  : Time when this application category was created.
    last_modified  : Time when this application category was last modified.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class ApplicationCategoryList(Collection):
    resource = ApplicationCategory
    resource_type = "application_categories"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

'''ArraysTestCase tests the Arrays object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Arrays TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Arrays TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_arrays(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().arrays.list(
        detail=True, limit=2)
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_iterate_arrays_endRow_beyond(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().arrays.get(endRow=10)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_select_fields_for_arrays(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().arrays.get(
            fields="name,pool_name,status,serial")
        assert resp is not None
        assert resp.attrs.get("name") is not None
        assert resp.attrs.get("pool_name") is not None
        assert resp.attrs.get("status") is not None
        assert resp.attrs.get("serial") is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_arrays(setup_teardown_for_each_test):
    nic_list = [
        {
            "subnet_label": "management",
            "data_ip": "127.0.0.23",
            "name": "eth1"
        },
        {
            "subnet_label": "management",
            "data_ip": "127.0.0.24",
            "name": "eth2"
        },
        {
            "subnet_label": "management",
            "data_ip": "127.0.0.25",
            "name": "eth3"
        },
        {
            "subnet_label": "management",
            "data_ip": "127.0.0.26",
            "name": "eth4"
        }
    ]

    serial = "g1a2"
    name = "g1a2"
    ctrlr_b_support_ip = "127.0.0.22"
    ctrlr_a_support_ip = "127.0.0.21"
    pool_name = "default"

    try:
        resp = nimosclientbase.get_nimos_client().arrays.create(
            name=name,
            ctrlr_a_support_ip=ctrlr_a_support_ip,
            ctrlr_b_support_ip=ctrlr_b_support_ip,
            pool_name=pool_name,
            nic_list=nic_list,
            serial=serial)
        assert resp is not None
        assert resp.attrs.get("description") == "modified by testcase"
    except exceptions.NimOSAPIError as ex:
        if "SM_enoent" in str(ex) or "SM_http_not_found" in str(ex):
            log("Failed as expected. no suc array object")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_check_mandatory_params_arrays(setup_teardown_for_each_test):
    serial = "g1a2"
    name = "g1a2"
    ctrlr_b_support_ip = "127.0.0.22"
    ctrlr_a_support_ip = "127.0.0.21"
    pool_name = "default"

    try:
        resp = nimosclientbase.get_nimos_client().arrays.create(
            name=name,
            ctrlr_a_support_ip=ctrlr_a_support_ip,
            ctrlr_b_support_ip=ctrlr_b_support_ip,
            pool_name=pool_name,
            # nic_list=nic_list,
            serial=serial)
        assert resp is not None
        assert resp.attrs.get("description") == "modified by testcase"
    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log("Failed as expected. some mandatory arguments missing")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

# global variables
app_server_name_1 = nimosclientbase.get_unique_string("appservertc-1")
app_server_to_delete = []


'''Appserver TestCase tests the app server object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for app_server  TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for app_server  TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    global app_server_name_1
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_app_server()
    # create a new unique name for next test case
    app_server_name_1 = nimosclientbase.get_unique_string("appservertc-1")
    nimosclientbase.log_footer(request.function.__name__)


def delete_app_server():
    for app_server_id in app_server_to_delete:
        nimosclientbase.get_nimos_client().application_servers.delete(
            app_server_id)
        log(f" Deleted app server user with id '{app_server_id}'")
    app_server_to_delete.clear()


def create_app_server(app_server, hostname="example.com", **kwargs):
    resp = nimosclientbase.get_nimos_client().application_servers.create(
        name=app_server, hostname=hostname, **kwargs)
    app_server_id = resp.attrs.get("id")
    app_server_to_delete.append(app_server_id)
    assert resp is not None
    log("Created app server with name "
        f"'{app_server}' with Id '{app_server_id}'")
    return resp


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_app_server(setup_teardown_for_each_test):
    resp = create_app_server(app_server_name_1)
    resp = nimosclientbase.get_nimos_client().application_servers.list(
        detail=True, limit=1)
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_create_app_server(setup_teardown_for_each_test):
    try:
        resp = create_app_server(
            app_server_name_1,
            hostname="example.com")
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_check_mandatory_params_app_server(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().application_servers.create(
            name=app_server_name_1)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log("Failed as expected. missing mandatory arguments.")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_app_servers_endrow_beyond(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().application_servers.get(
            endRow=30)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected.no rows present")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_select_fields_for_app_server(setup_teardown_for_each_test):
    try:
        resp = create_app_server(app_server_name_1, hostname="example.com")
        resp = nimosclientbase.get_nimos_client().application_servers.get(
            fields="name,hostname,port,creation_time")
        assert resp is not None
        assert resp.attrs.get("name") is not None
        assert resp.attrs.get("hostname") is not None
        assert resp.attrs.get("port") is not None
        assert resp.attrs.get("creation_time") is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_app_server(setup_teardown_for_each_test):
    try:
        resp = create_app_server(app_server=app_server_name_1)
        assert resp is not None
        # update few fields
        update_resp = nimosclientbase.get_nimos_client(
        ).application_servers.update(
            id=resp.attrs.get("id"),
            name="updatedname",
            description="modified by testcase",
            username="abc")
        update_resp is not None
        # assert the values got updated
        assert update_resp.attrs.get("name") == "updatedname"
        assert update_resp.attrs.get("description") == "modified by testcase"
        assert update_resp.attrs.get("username") == "abc"
    except exceptions.NimOSAPIError as ex:
        if "SM_end_row_beyond_total_rows" in str(ex):
            log("Failed as expected")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class UserGroup(Resource):
    """Represents Active Directory groups configured to manage the system.

    # Parameters
    id                 : Identifier for the user group.
    name               : Name of the user group.
    description        : Description of the user group.
    role_id            : Identifier for the user group's role.
    role               : Role of the user.
    inactivity_timeout : The amount of time that the user session is inactive before timing out. A value of 0 indicates that the timeout is taken from the group setting.
    creation_time      : Time when this user was created.
    last_modified      : Time when this user was last modified.
    disabled           : User is currently disabled.
    external_id        : External ID of the user group. In Active Directory, it is the group's SID (Security Identifier).
    domain_id          : Identifier of the domain this user group belongs to.
    domain_name        : Role of the user.
    """


class UserGroupList(Collection):
    resource = UserGroup
    resource_type = "user_groups"



# pytest_plugins = 'pytest_session2file'
# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Pytest Plugin that save failure or test session information to a file pass as
a command line argument to pytest.

It put in a file exactly what pytest return to the stdout.

To use it :
Put this file in the root of tests/ edit your conftest and insert in the top
of the file :

    pytest_plugins = 'pytest_session_to_file'

Then you can launch your test with the new option --session_to_file=like this :

    py.test --session_to_file=FILENAME
Or :
    py.test -p pytest_session_to_file --session_to_file=FILENAME


Inspire by _pytest.pastebin
Ref: https://github.com/pytest-dev/pytest/blob/master/_pytest/pastebin.py

Version : 0.1
Date : 30 sept. 2015 11:25
Copyright (C) 2015 Richard Vézina <ml.richard.vezinar @ gmail.com>
Licence : Public Domain
"""

import pytest
import tempfile


def pytest_addoption(parser):
    group = parser.getgroup("terminal reporting")
    group._addoption('--session_to_file', action='store', metavar='path',
                     default='tests//logs//pytest_session.log',
                     help="Save to file the pytest session information")


@pytest.hookimpl(trylast=True)
def pytest_configure(config):
    tr = config.pluginmanager.getplugin('terminalreporter')
    # if no terminal reporter plugin is present, nothing we can do here;
    # this can happen when this function executes in a slave node
    # when using pytest-xdist, for example
    if tr is not None:
        config._pytestsessionfile = tempfile.TemporaryFile('w+')
        oldwrite = tr._tw.write

        def tee_write(s, **kwargs):
            oldwrite(s, **kwargs)
            config._pytestsessionfile.write(str(s))
        tr._tw.write = tee_write


def pytest_unconfigure(config):
    if hasattr(config, '_pytestsessionfile'):
        # get terminal contents and delete file
        config._pytestsessionfile.seek(0)
        sessionlog = config._pytestsessionfile.read()
        config._pytestsessionfile.close()
        del config._pytestsessionfile
        # undo our patching in the terminal reporter
        tr = config.pluginmanager.getplugin('terminalreporter')
        del tr._tw.__dict__['write']
        # write summary
        create_new_file(config=config, contents=sessionlog)


def create_new_file(config, contents):
    """
    Creates a new file with pytest session contents.
    :contents: paste contents
    :returns: url to the pasted contents
    """
    # import _pytest.config
    # path = _pytest.config.option.session_to_file
    # path = 'pytest_session.txt'
    try:
        path = config.option.session_to_file
        with open(path, 'w') as f:
            f.writelines(contents)
    except Exception:
        pass


def pytest_terminal_summary(terminalreporter):
    import _pytest.config
    tr = terminalreporter
    if 'failed' in tr.stats:
        for rep in terminalreporter.stats.get('failed'):
            try:
                rep.longrepr.reprtraceback.reprentries[-1].reprfileloc
            except AttributeError:
                tr._getfailureheadline(rep)
            tw = _pytest.config.create_terminal_writer(terminalreporter.config,
                                                       stringio=True)
            rep.toterminal(tw)
            s = tw.stringio.getvalue()
            assert len(s)
            create_new_file(config=_pytest.config, contents=s)



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#

import nimbleclient.v1

from .query import (
    and_,
    or_
)

# Version for NimbleOS SDK client package
__version__ = "1.1.0"

__all__ = [
    "v1",
    "and_",
    "or_",
    "NimOSClient",
]


def NimOSClient(hostname, username, password, job_timeout=60, port=5392, version=1):
    """Instantiates Nimble client instance to interact with the NimOS REST API Server."""

    return getattr(nimbleclient, f'v{version}').NimOSClient(hostname, username, password, job_timeout, port)



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class AccessControlRecord(Resource):
    """Manage access control records for volumes.

    # Parameters
    id                   : Identifier for the access control record.
    apply_to             : Type of object this access control record applies to.
    chap_user_id         : Identifier for the CHAP user.
    chap_user_name       : Name of the CHAP user.
    initiator_group_id   : Identifier for the initiator group.
    initiator_group_name : Name of the initiator group.
    lun                  : If this access control record applies to a regular volume, this attribute is the volume's LUN (Logical Unit Number). If the access protocol is iSCSI,
                           the LUN will be 0. However, if the access protocol is Fibre Channel, the LUN will be in the range from 0 to 2047. If this record applies to a Virtual
                           Volume, this attribute is the volume's secondary LUN in the range from 0 to 399999, for both iSCSI and Fibre Channel. If the record applies to a
                           OpenstackV2 volume, the LUN will be in the range from 0 to 2047, for both iSCSI and Fibre Channel. If this record applies to a protocol endpoint or only
                           a snapshot, this attribute is not meaningful and is set to null.
    vol_id               : Identifier for the volume this access control record applies to.
    vol_name             : Name of the volume this access control record applies to.
    vol_agent_type       : External management agent type.
    pe_id                : Identifier for the protocol endpoint this access control record applies to.
    pe_name              : Name of the protocol endpoint this access control record applies to.
    pe_lun               : LUN (Logical Unit Number) to associate with this protocol endpoint. Valid LUNs are in the 0-2047 range.
    snap_id              : Identifier for the snapshot this access control record applies to.
    snap_name            : Name of the snapshot this access control record applies to.
    pe_ids               : List of candidate protocol endpoints that may be used to access the Virtual Volume. One of them will be selected for the access control record. This
                           field is required only when creating an access control record for a Virtual Volume.
    snapluns             : Information about the snapshot LUNs associated with this access control record. This field is meaningful when the online snapshot can be accessed as a
                           LUN in the group.
    creation_time        : Time when this access control record was created.
    last_modified        : Time when this access control record was last modified.
    access_protocol      : Access protocol of the volume.
    """

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class AccessControlRecordList(Collection):
    resource = AccessControlRecord
    resource_type = "access_control_records"

    def create(self, **kwargs):
        resp = self._client.create_resource(self.resource_type, **kwargs)
        return self.resource(resp['id'], resp, client=self._client, collection=self)

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

from tests.nimbleclientbase import SKIPTEST, log_to_file as log
import tests.nimbleclientbase as nimosclientbase
import pytest

'''SubnetTestCase tests the subnet object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Subnet Testcase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for Subnet Testcase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture
def setup_teardown(before_running_all_testcase, request):
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown  # provide the fixture value
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_subnet_details(setup_teardown):
    # sdk bug. why is subnet object having functions like create,update
    # delete??? the rest doc does not have these. only read is allowed
    resp = nimosclientbase.get_nimos_client().subnets.get()
    assert resp is not None
    # doc shows it has 13 properties.but in replication setup it is 15.
    # just check the length
    assert(resp.attrs.__len__() >= 13)



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Witness(Resource):
    """Manage witness host configuration.

    # Parameters
    id                       : Identifier of the witness configuration.
    username                 : Username of witness. This has to be a non-root that can login to the witness host.
    password                 : Password of witness.
    host                     : Hostname or ip addresses of witness.
    port                     : Port of witness.
    secure_mode              : To verify the witness host against CA cert and to apply possible security modes.
    auto_switchover_messages : List of validation messages for automatic switchover of Group Management. This will be empty when there are no conflicts found.
    """
    def test(self, **kwargs):
        """Tests and validates witness configuration between the array and the witness.

        # Parameters
        id : ID of the witness.
        """

        return self._collection.test(
            self.id,
            **kwargs
        )

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class WitnessList(Collection):
    resource = Witness
    resource_type = "witnesses"

    def test(self, id, **kwargs):
        """Tests and validates witness configuration between the array and the witness.

        # Parameters
        id : ID of the witness.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'test',
            id=id,
            **kwargs
        )

    def create(self, **kwargs):
        resp = self._client.create_resource(self.resource_type, **kwargs)
        return self.resource(resp['id'], resp, client=self._client, collection=self)

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

'''NetworkconfigTestCase tests the Networkconfig object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for NetworkConfig TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for NetworkConfig TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


route_list = [
    {
        "gateway": "127.0.0.1",
        "tgt_network": "0.0.0.0",
        "tgt_netmask": "0.0.0.0"
    }
]
subnet_list = [
    {
        "label": "subnet1",
        "network": "127.0.0.0",
        "netmask": "255.0.0.0",
        "type": "mgmt",
        "allow_iscsi": False,
        "allow_group": False,
        "netzone_type": "single",
        "discovery_ip": "127.0.0.102",
        "mtu": 1500,
        "vlan_id": 0
    }
]

array_list = [
    {
        "name": "g1a1",
        # "member_gid": 10,
        "ctrlr_a_support_ip": "127.0.0.11",
        "ctrlr_b_support_ip": "127.0.0.21",
        "nic_list": [
            {
                "name": "eth1",
                "subnet_label": "subnet1",
                "data_ip": "127.0.0.91",
                "tagged": True
            }
        ]
    },
    {
        "name": "g1a2",
        # "member_gid": 11,
        "ctrlr_a_support_ip": "127.0.0.12",
        "ctrlr_b_support_ip": "127.0.0.22",
        "nic_list": [
            {
                "name": "eth1",
                "subnet_label": "subnet1",
                "data_ip": "127.0.0.92",
                "tagged": False
            }
        ]
    }

]


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_network_configs_details(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().network_configs.list()
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_check_mandatory_params_network_configs(setup_teardown_for_each_test):
    try:
        resp = nimosclientbase.get_nimos_client().network_configs.create(
            name="draft",
            mgmt_ip="127.0.0.1",
            iscsi_automatic_connection_method=False,
            iscsi_connection_rebalancing=False,
            route_list=route_list,
            subnet_list=subnet_list,
            array_list=array_list
        )
        assert resp is not None
        # the create will in any case fail as the mgmt ip is not correct but
        # atleast we should always get exception "sm_array_not_found"
        # this exception only comes when all the mandatory params are
        # atleast present.values may be incorrect for those param
    except exceptions.NimOSAPIError as ex:
        if "SM_array_not_found" in str(ex):
            log("Failed as expected")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Job(Resource):
    """Jobs are operations in progress in the system.

    # Parameters
    completion_time           : Completion time of the job.
    creation_time             : Time when this job was created.
    current_phase             : Phase number of the job in progress.
    current_phase_description : Description of the current phase of the job.
    description               : Description of the job.
    id                        : Identifier for job.
    name                      : Name of the job.
    last_modified             : Time of the last update from the job.
    object_id                 : Identifier for object being acted upon.
    op_type                   : Type of operation.
    type                      : Job type.
    parent_job_id             : Identifier of parent job.
    percent_complete          : Progress of the job as a percentage.
    request                   : Original request that the job is responsible for.
    response                  : Response from the operation as the job executes.
    state                     : Status of the job.
    result                    : Result of the job.
    total_phases              : Total number of phases of the job.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class JobList(Collection):
    resource = Job
    resource_type = "jobs"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions
import tests.test_volume as volume
import threading

# global variables
volcoll_name1 = nimosclientbase.get_unique_string("volcolltc-volcoll1")
volcoll_to_delete = []
volcoll_lock = threading.RLock()

'''VolumeCollectionTestCase tests the volumecollection object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for VolumeCollectionTestCase *****\n")
    cleanup_old_volume_collection()

    def after_running_all_testcase():
        log("**** Completed Tests for VolumeCollectionTestCase "
            "Testcase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    global volcoll_name1
    volcoll_name1 = nimosclientbase.get_unique_string("volcolltc-volcoll1")
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    volume.delete_volume()
    delete_volcoll()
    nimosclientbase.log_footer(request.function.__name__)


def cleanup_old_volume_collection():
    log("Cleaning up any unwanted leftover volcoll from previous run")
    volcoll_resp = nimosclientbase.get_nimos_client().volume_collections.list(detail=True)
    for volcoll_obj in volcoll_resp:
        try:
            if (str.startswith(volcoll_obj.attrs.get("name"), "snapcolltc-vol")):
                volcoll_name = volcoll_obj.attrs.get("name")
                resp = nimosclientbase.get_nimos_client(
                ).volume_collections.delete(id=volcoll_obj.attrs.get("id"))
                assert resp is not None
                log(f" Deleted volcoll with name '{volcoll_name}'")
        except Exception as ex:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


def delete_volcoll():
    volcoll_lock.acquire()
    for volcoll_id in volcoll_to_delete:
        try:
            resp = nimosclientbase.get_nimos_client(
            ).volume_collections.delete(id=volcoll_id)
            assert resp is not None
            log(f" Deleted volcoll with id '{volcoll_id}'")
        except exceptions.NimOSAPIError as ex:
            volcoll_lock.release()
            log(f"Failed with exception message : {str(ex)}")
            raise ex
    volcoll_to_delete.clear()
    volcoll_lock.release()


def create_volcoll(volcollname):
    try:
        volcoll_lock.acquire()
        resp = nimosclientbase.get_nimos_client().volume_collections.create(
            name=volcollname, description="created by testcase")
        volcoll_id = resp.attrs.get("id")
        volcoll_to_delete.append(volcoll_id)
        assert resp is not None
        log(f"Created volcoll with name '{volcollname}' and Id '{volcoll_id}'")
        volcoll_lock.release()
        return resp
    except Exception as ex:
        volcoll_lock.release()
        log(f"Failed with exception message : {str(ex)}")
        raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_create_volcoll(setup_teardown_for_each_test):
    resp = create_volcoll(volcoll_name1)
    assert resp is not None
    assert volcoll_name1 == resp.attrs.get("name")
    assert "created by testcase" == resp.attrs.get("description")

    # change the description and test it works
    resp = nimosclientbase.get_nimos_client().volume_collections.update(
        resp.attrs.get("id"), description="modified by testcase")
    assert resp is not None
    assert "modified by testcase" == resp.attrs.get("description")


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_add_volume_to_volcoll(setup_teardown_for_each_test):
    volcoll_resp = create_volcoll(volcoll_name1)
    voloume_name = nimosclientbase.get_unique_string(
        "VolumeCollectionTestCase-addtovolcoll")

    vol_resp = nimosclientbase.get_nimos_client().volumes.create(
        voloume_name, size=50)
    volume.vol_to_delete.append(vol_resp.attrs.get("id"))
    assert volcoll_resp is not None

    # associate the volume to volcoll
    vol_associate_resp = nimosclientbase.get_nimos_client().volumes.associate(
        id=vol_resp.attrs.get("id"), volcoll=volcoll_resp)
    assert vol_resp is not None
    # check
    assert vol_associate_resp.get("volcoll_name") == volcoll_name1
    assert vol_associate_resp.get("volcoll_id") == volcoll_resp.attrs.get("id")
    # disassociate
    nimosclientbase.get_nimos_client().volumes.dissociate(
        id=vol_resp.attrs.get("id"))
    # get vol coll and confirm has no volumes
    volcoll_resp = nimosclientbase.get_nimos_client(
    ).volume_collections.get(id=volcoll_resp.attrs.get("id"))
    assert volcoll_resp.attrs.get("volume_count") == 0


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_delete_volcoll_before_disassociating_volume(
        setup_teardown_for_each_test):
    volcoll_resp = create_volcoll(volcoll_name1)
    voloume_name = nimosclientbase.get_unique_string(
        "VolumeCollectionTestCase-addtovolcoll")

    vol_resp = nimosclientbase.get_nimos_client().volumes.create(
        voloume_name, size=50)
    volume.vol_to_delete.append(vol_resp.attrs.get("id"))
    assert volcoll_resp is not None

    # associate the volume to volcoll
    vol_associate_resp = nimosclientbase.get_nimos_client().volumes.associate(
        id=vol_resp.attrs.get("id"), volcoll=volcoll_resp)
    assert vol_resp is not None
    # check
    assert vol_associate_resp.get("volcoll_name") == volcoll_name1
    assert vol_associate_resp.get("volcoll_id") == volcoll_resp.attrs.get("id")
    # try deleting the volcoll . this should fail as volume has
    # not been disassocited
    try:
        nimosclientbase.get_nimos_client().volume_collections.delete(
            id=volcoll_resp.attrs.get("id"))
    except exceptions.NimOSAPIError as ex:
        if"SM_ebusy" in str(ex):
            log("Failed as expected. disaasociate volume first")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_promote_volcoll(setup_teardown_for_each_test):
    volcoll_resp = create_volcoll(volcoll_name1)
    voloume_name = nimosclientbase.get_unique_string(
        "VolumeCollectionTestCase-addtovolcoll")

    vol_resp = nimosclientbase.get_nimos_client().volumes.create(
        voloume_name, size=50)
    volume.vol_to_delete.append(vol_resp.attrs.get("id"))
    assert volcoll_resp is not None

    # associate the volume to volcoll
    vol_associate_resp = nimosclientbase.get_nimos_client().volumes.associate(
        id=vol_resp.attrs.get("id"), volcoll=volcoll_resp)
    assert vol_resp is not None
    # check
    assert vol_associate_resp.get("volcoll_name") == volcoll_name1
    assert vol_associate_resp.get("volcoll_id") == volcoll_resp.attrs.get("id")
    try:
        nimosclientbase.get_nimos_client().volume_collections.promote(
            id=volcoll_resp.attrs.get("id"))
    except exceptions.NimOSAPIError as ex:
        if"SM_ealready" in str(ex):
            log("Failed as expected. volcoll is already promoted")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_demote_volcoll(setup_teardown_for_each_test):
    volcoll_resp = create_volcoll(volcoll_name1)
    assert volcoll_resp is not None
    try:
        nimosclientbase.get_nimos_client().volume_collections.demote(
            id=volcoll_resp.attrs.get("id"),
            replication_partner_id="1264126491231239123hgghsjhd")
    except exceptions.NimOSAPIError as ex:
        if"SM_invalid_arg_value" in str(ex):
            log("Failed as expected. "
                "Invalid value provided for replication_partner_id")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_handover_volcoll(setup_teardown_for_each_test):
    volcoll_resp = create_volcoll(volcoll_name1)
    assert volcoll_resp is not None
    try:
        nimosclientbase.get_nimos_client().volume_collections.handover(
            id=volcoll_resp.attrs.get("id"),
            replication_partner_id="1264126491231239123hgghsjhd")
    except exceptions.NimOSAPIError as ex:
        if"SM_invalid_arg_value" in str(ex):
            log("Failed as expected. "
                "Invalid value provided for replication_partner_id")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_delete_volume_in_volcoll_before_disassociating(
        setup_teardown_for_each_test):

    volcoll_resp = create_volcoll(volcoll_name1)
    voloume_name = nimosclientbase.get_unique_string(
        "VolumeCollectionTestCase-addtovolcoll")

    vol_resp = nimosclientbase.get_nimos_client().volumes.create(
        voloume_name, size=50)
    volume.vol_to_delete.append(vol_resp.attrs.get("id"))
    assert volcoll_resp is not None

    # associate the volume to volcoll
    vol_associate_resp = nimosclientbase.get_nimos_client().volumes.associate(
        id=vol_resp.attrs.get("id"), volcoll=volcoll_resp)
    assert vol_resp is not None
    # check
    assert vol_associate_resp.get("volcoll_name") == volcoll_name1
    assert vol_associate_resp.get("volcoll_id") == volcoll_resp.attrs.get("id")
    # try deleting the volcoll . this should fail as
    # volume has not been disassocited
    try:
        nimosclientbase.get_nimos_client().volumes.offline(
            id=vol_resp.attrs.get("id"))
        nimosclientbase.get_nimos_client().volumes.delete(
            id=vol_resp.attrs.get("id"))
    except exceptions.NimOSAPIError as ex:
        if"SM_vol_assoc_volcoll" in str(ex):
            log("Failed as expected with exception SM_vol_assoc_volcoll")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_create_edit_delete_protection_schedule(setup_teardown_for_each_test):
    # check if this array has any previous volcoll
    previous_volcoll_resp = nimosclientbase.get_nimos_client(
    ).volume_collections.list()
    total_prev_volcoll = previous_volcoll_resp.__len__()

    protection_sched_name = "testcaseprotectionschedule"
    days = "monday,tuesday,wednesday,thursday,friday"
    description = "super cool schedule"

    volcoll_resp = create_volcoll(volcoll_name1)
    assert volcoll_resp is not None
    # create a protection schedule
    protect_sched_resp = nimosclientbase.get_nimos_client(
    ).protection_schedules.create(
        name=protection_sched_name,
        days=days, description=description,
        volcoll_or_prottmpl_id=volcoll_resp.attrs.get("id"),
        volcoll_or_prottmpl_type='volume_collection',
        num_retain=2)
    assert protect_sched_resp is not None
    assert protect_sched_resp.attrs.get("days") == days
    assert protect_sched_resp.attrs.get("description") == description
    assert protect_sched_resp.attrs.get("name") == protection_sched_name
    # check if volcoll is present
    cur_volcoll_resp = nimosclientbase.get_nimos_client(
    ).volume_collections.list()
    assert cur_volcoll_resp.__len__() >= total_prev_volcoll

    # update the schedule
    resp = nimosclientbase.get_nimos_client().protection_schedules.update(
        id=protect_sched_resp.attrs.get("id"), period_unit="minutes")
    assert resp is not None
    assert "minutes" == resp.attrs.get("period_unit")
    # delete the schedule
    resp = nimosclientbase.get_nimos_client().protection_schedules.delete(
        id=protect_sched_resp.attrs.get("id"))
    # check if volcoll has schedule
    volcoll_resp = nimosclientbase.get_nimos_client().volume_collections.get(
        id=volcoll_resp.attrs.get("id"))
    assert volcoll_resp.attrs.get("schedule_list") is None



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log


'''FCTestCases tests the fibre channel functionality. it covers
    ports,session, initiator_aliases and interface object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for FibreChannel ports and "
        "session TestCases *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for FibreChannel ports "
            "and session TestCases *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_fc_ports(setup_teardown_for_each_test):
    nimosclientbase.get_nimos_client().fibre_channel_ports.get()


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_fc_sessions(setup_teardown_for_each_test):
    nimosclientbase.get_nimos_client().fibre_channel_sessions.get()


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_fc_interfaces(setup_teardown_for_each_test):
    nimosclientbase.get_nimos_client().fibre_channel_interfaces.get()


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_fc_initiators_aliases(setup_teardown_for_each_test):
    nimosclientbase.get_nimos_client().fibre_channel_initiator_aliases.get()



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP
#
# @author alok ranjan
from tests.nimbleclientbase import log_to_file as log

'''AD_MembershipTestCase is not implemented due to external dependency '''

log("**** AD_MembershipTestCase is not implemented due to "
    "external dependency *****'")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class InitiatorGroup(Resource):
    """Manage initiator groups for initiator authentication. An initiator group is a set of initiators that can be configured as part of your ACL to access a specific volume through
    group membership.

    # Parameters
    id               : Identifier for initiator group.
    name             : Name of initiator group.
    full_name        : Initiator group's full name.
    search_name      : Initiator group name used for search.
    description      : Text description of initiator group.
    access_protocol  : Initiator group access protocol.
    host_type        : Initiator group host type. Available options are auto and hpux. The default option is auto. This attribute will be applied to all the initiators in the
                       initiator group. Initiators with different host OSes should not be kept in the same initiator group having a non-default host type attribute.
    fc_tdz_ports     : List of target Fibre Channel ports with Target Driven Zoning configured on this initiator group.
    target_subnets   : List of target subnet labels. If specified, discovery and access to volumes will be restricted to the specified subnets.
    iscsi_initiators : List of iSCSI initiators. When create/update iscsi_initiators, either iqn or ip_address is always required with label.
    fc_initiators    : List of FC initiators. When create/update fc_initiators, wwpn is required.
    creation_time    : Time when this initiator group was created.
    last_modified    : Time when this initiator group was last modified.
    vp_override      : Flag to allow modifying VP created initiator groups. When set to true, user can add this initiator to a VP created initiator group.
    app_uuid         : Application identifier of initiator group.
    volume_count     : Number of volumes that are accessible by the initiator group.
    volume_list      : List of volumes that are accessible by the initiator group.
    num_connections  : Total number of connections from initiators in the initiator group.
    metadata         : Key-value pairs that augment an initiator group's attributes.
    """
    def suggest_lun(self, **kwargs):
        """Suggest an LU number for the volume and initiator group combination.

        # Parameters
        id     : ID of the initiator group.
        vol_id : ID of the volume.
        """

        return self._collection.suggest_lun(
            self.id,
            **kwargs
        )
    def validate_lun(self, lun, **kwargs):
        """Validate an LU number for the volume and initiator group combination.

        # Parameters
        id     : ID of the initiator group.
        vol_id : ID of the volume.
        lun    : LU number to validate in decimal.
        """

        return self._collection.validate_lun(
            self.id,
            lun,
            **kwargs
        )


class InitiatorGroupList(Collection):
    resource = InitiatorGroup
    resource_type = "initiator_groups"

    def suggest_lun(self, id, **kwargs):
        """Suggest an LU number for the volume and initiator group combination.

        # Parameters
        id     : ID of the initiator group.
        vol_id : ID of the volume.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'suggest_lun',
            id=id,
            **kwargs
        )

    def validate_lun(self, id, lun, **kwargs):
        """Validate an LU number for the volume and initiator group combination.

        # Parameters
        id     : ID of the initiator group.
        vol_id : ID of the volume.
        lun    : LU number to validate in decimal.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'validate_lun',
            id=id,
            lun=lun,
            **kwargs
        )



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

'''audit_logTestCase tests the audit_log object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for audit_log TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for audit_log TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_audit_log(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().audit_log.get(limit=1)
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_audit_log_query_params(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().audit_log.list(
        detail=True,
        limit=2)
    assert resp is not None
    resp = nimosclientbase.get_nimos_client().audit_log.get(
        limit=2,
        fields="user_name,id,status")
    # assert that those fields are present
    assert resp.attrs.get("user_name") is not None
    assert resp.attrs.get("id") is not None
    assert resp.attrs.get("status") is not None
    # try asserting a value which was not querying
    assert resp.attrs.get("startRow") is None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_list_audit_logs(setup_teardown_for_each_test):
    resp = nimosclientbase.get_nimos_client().audit_log.list(limit=5)
    limit = 500
    while 1:
        resp = nimosclientbase.get_nimos_client().audit_log.list(limit=limit, from_id=resp[len(resp)-1].attrs.get("id"))
        if len(resp) < limit:
            break
    assert resp is not None


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_query_invalid_params(setup_teardown_for_each_test):
    try:
        query_param = "junkparam"
        resp = nimosclientbase.get_nimos_client().audit_log.get(
            limit=2,
            fields=query_param)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_query_param" in str(ex):
            log("Failed as expected. "
                f"Invalid query params provided to query '{query_param}'")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Alarm(Resource):
    """View alarms.

    # Parameters
    id                     : Identifier for the alarm.
    type                   : Identifier for type of alarm.
    array                  : The array name where the alarm is generated.
    curr_onset_event_id    : Identifier for the current onset event.
    object_id              : Identifier of object operated upon.
    object_name            : Name of object operated upon.
    object_type            : Type of the object being operated upon.
    onset_time             : Time when this alarm was triggered.
    ack_time               : Time when this alarm was acknowledged.
    status                 : Status of the operation -- open or acknowledged.
    user_id                : Identifier of the user who acknowledged the alarm.
    user_name              : Username of the user who acknowledged the alarm.
    user_full_name         : Full name of the user who acknowledged the alarm.
    category               : Category of the alarm.
    severity               : Severity level of the event.
    remind_every           : Frequency of notification. This number and the remind_every_unit define how frequent one alarm notification is sent. For example, a value of 1 with
                             the 'remind_every_unit' of 'days' results in one notification every day.
    remind_every_unit      : Time unit over which to send the number of notification specified in 'remind_every'. For example, a value of 'days' with a 'remind_every' of '1'
                             results in one notification every day.
    activity               : Description of activity performed and recorded in alarm.
    next_notification_time : Time when next reminder for the alarm will be sent.
    """
    def acknowledge(self, **kwargs):
        """Acknowledge an alarm.

        # Parameters
        id                : ID of the acknowledged alarm.
        remind_every      : Notification frequency unit.
        remind_every_unit : Period unit.
        """

        return self._collection.acknowledge(
            self.id,
            **kwargs
        )
    def unacknowledge(self, **kwargs):
        """Unacknowledge an alarm.

        # Parameters
        id : ID of the acknowledged alarm.
        """

        return self._collection.unacknowledge(
            self.id,
            **kwargs
        )

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")


class AlarmList(Collection):
    resource = Alarm
    resource_type = "alarms"

    def acknowledge(self, id, **kwargs):
        """Acknowledge an alarm.

        # Parameters
        id                : ID of the acknowledged alarm.
        remind_every      : Notification frequency unit.
        remind_every_unit : Period unit.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'acknowledge',
            id=id,
            **kwargs
        )

    def unacknowledge(self, id, **kwargs):
        """Unacknowledge an alarm.

        # Parameters
        id : ID of the acknowledged alarm.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'unacknowledge',
            id=id,
            **kwargs
        )

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class UserPolicy(Resource):
    """Manages the password policies configured for the group.

    # Parameters
    id               : Identifier for the security policy.
    allowed_attempts : Number of authentication attempts allowed before the user account is locked. Allowed range is [1, 10] inclusive. '0' indicates no limit.
    min_length       : Minimum length for user passwords. Allowed range is [8, 255] inclusive.
    upper            : Number of uppercase characters required in user passwords. Allowed range is [0, 255] inclusive.
    lower            : Number of lowercase characters required in user passwords. Allowed range is [0, 255] inclusive.
    digit            : Number of numerical characters required in user passwords. Allowed range is [0, 255] inclusive.
    special          : Number of special characters required in user passwords. Allowed range is [0, 255] inclusive.
    previous_diff    : Number of characters that must be different from the previous password. Allowed range is [1, 255] inclusive.
    no_reuse         : Number of times that a password must change before you can reuse a previous password. Allowed range is [1,20] inclusive.
    max_sessions     : Maximum number of sessions allowed for a group. Allowed range is [10, 1000] inclusive. '0' indicates no limit.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")


class UserPolicyList(Collection):
    resource = UserPolicy
    resource_type = "user_policies"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class AuditLog(Resource):
    """View audit log.

    # Parameters
    id                  : Identifier for the audit log record.
    type                : Identifier for type of audit log record.
    object_id           : Identifier of object operated upon.
    object_name         : Name of object operated upon.
    object_type         : Type of the object being operated upon.
    scope               : Scope within which object exists, for example, name of the array for a NIC.
    time                : Time when this operation was performed.
    status              : Status of the operation -- success or failure.
    error_code          : If the operation has failed, this indicates the error code corresponding to the failure.
    user_id             : Identifier of the user who performed the operation.
    user_name           : Username of the user who performed the operation.
    user_full_name      : Full name of the user who performed the operation.
    source_ip           : IP address from where the operation request originated.
    ext_user_id         : The user id of an external user.
    ext_user_group_id   : The group ID of an external user.
    ext_user_group_name : The group name of an external user.
    app_name            : Name of application from where the operation request was issued, for example, pam, VSS Agent, etc.
    access_type         : Name of access on how the operation request was issued, for example, GUI, CLI or API.
    category            : Category of the audit log record.
    activity_type       : Type of activity performed, for example, create, update or delete.
    activity            : Description of activity performed and recorded in audit log.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class AuditLogList(Collection):
    resource = AuditLog
    resource_type = "audit_log"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class ActiveDirectoryMembership(Resource):
    """Manages the storage array's membership with the Active Directory.

    # Parameters
    id                  : Identifier for the Active Directory Domain.
    description         : Description for the Active Directory Domain.
    name                : Identifier for the Active Directory domain.
    netbios             : Netbios name for the Active Directory domain.
    server_list         : List of IP addresses or names for the backup domain controller.
    computer_name       : The name of the computer account in the domain controller.
    organizational_unit : The location for the computer account.
    user                : Name of the Activer Directory user with Administrator's privilege.
    password            : Password for the Active Directory user.
    enabled             : Active Directory authentication is currently enabled.
    """
    def remove(self, password, user, **kwargs):
        """Leaves the Active Directory domain.

        # Parameters
        id       : ID of the active directory.
        user     : Name of the Activer Directory user with the privilege to leave the domain.
        password : Password for the Active Directory user.
        force    : Use this option when there is an error when leaving the domain.
        """

        return self._collection.remove(
            self.id,
            password,
            user,
            **kwargs
        )
    def report_status(self, **kwargs):
        """Reports the detail status of the Active Directory domain.

        # Parameters
        id : ID of the active directory.
        """

        return self._collection.report_status(
            self.id,
            **kwargs
        )
    def test_user(self, name, **kwargs):
        """Tests whether the user exist in the Active Directory. If the user is present, then the user's group and role information is reported.

        # Parameters
        id   : ID of the Active Directory.
        name : Name of the Active Directory user.
        """

        return self._collection.test_user(
            self.id,
            name,
            **kwargs
        )
    def test_group(self, name, **kwargs):
        """Tests whether the user group exist in the Active Directory.

        # Parameters
        id   : ID of the Active Directory.
        name : Name of the Active Directory group.
        """

        return self._collection.test_group(
            self.id,
            name,
            **kwargs
        )

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")


class ActiveDirectoryMembershipList(Collection):
    resource = ActiveDirectoryMembership
    resource_type = "active_directory_memberships"

    def remove(self, id, password, user, **kwargs):
        """Leaves the Active Directory domain.

        # Parameters
        id       : ID of the active directory.
        user     : Name of the Activer Directory user with the privilege to leave the domain.
        password : Password for the Active Directory user.
        force    : Use this option when there is an error when leaving the domain.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'remove',
            id=id,
            password=password,
            user=user,
            **kwargs
        )

    def report_status(self, id, **kwargs):
        """Reports the detail status of the Active Directory domain.

        # Parameters
        id : ID of the active directory.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'report_status',
            id=id,
            **kwargs
        )

    def test_user(self, id, name, **kwargs):
        """Tests whether the user exist in the Active Directory. If the user is present, then the user's group and role information is reported.

        # Parameters
        id   : ID of the Active Directory.
        name : Name of the Active Directory user.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'test_user',
            id=id,
            name=name,
            **kwargs
        )

    def test_group(self, id, name, **kwargs):
        """Tests whether the user group exist in the Active Directory.

        # Parameters
        id   : ID of the Active Directory.
        name : Name of the Active Directory group.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'test_group',
            id=id,
            name=name,
            **kwargs
        )

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection


class ReplicationPartner(Resource):
    """Manage replication partner. Replication partners let one storage array talk to another for replication purposes. The two arrays must be able to communicate over a network, and
    use ports 4213 and 4214. Replication partners have the same name as the remote group. Replication partners can be reciprocal, upstream (the source of replicas), or downstream
    (the receiver of replicas) partners.

    # Parameters
    id                               : Identifier for a replication partner.
    name                             : Name of replication partner.
    full_name                        : Fully qualified name of replication partner.
    search_name                      : Name of replication partner used for object search.
    description                      : Description of replication partner.
    partner_type                     : Replication partner type. Possible values are group or pool.
    alias                            : Name this group uses to identify itself to this partner.
    secret                           : Replication partner shared secret, used for mutual authentication of the partners.
    creation_time                    : Time when this replication partner was created.
    last_modified                    : Time when this replication partner was last modified.
    control_port                     : Port number of partner control interface.
    hostname                         : IP address or hostname of partner interface.  This must be the partner's Group Management IP address.
    port_range_start                 : For tunnel_endpoint partner types, first port available on the ssh proxy available for reverse forwarding. It must be guaranteed that the
                                       proxy has the next N ports reserved for this partner, where N is the count of DSDs in this group. This attribute is only valid for
                                       tunnel_endpoint partner type.
    proxy_hostname                   : IP address of tunnel endpoint. Only valid for tunnel_endpoint partner types.
    proxy_user                       : User to authenticate with tunnel endpoint. Only valid for tunnel_endpoint partner types.
    repl_hostname                    : IP address or hostname of partner data interface.
    data_port                        : Port number of partner data interface.
    is_alive                         : Whether the partner is available, and responding to pings.
    partner_group_uid                : Replication partner group uid.
    last_keepalive_error             : Most recent error while attempting to ping the partner.
    cfg_sync_status                  : Indicates whether all volumes and volume collections have been synced to the partner.
    last_sync_error                  : Most recent error seen while attempting to sync objects to the partner.
    array_serial                     : Serial number of group leader array of the partner.
    version                          : Replication version of the partner.
    pool_id                          : The pool ID where volumes replicated from this partner will be created. Replica volumes created as clones ignore this parameter and are
                                       always created in the same pool as their parent volume.
    pool_name                        : The pool name where volumes replicated from this partner will be created.
    folder_id                        : The Folder ID within the pool where volumes replicated from this partner will be created. This is not supported for pool partners.
    folder_name                      : The Folder name within the pool where volumes replicated from this partner will be created.
    match_folder                     : Indicates whether to match the upstream volume's folder on the downstream.
    paused                           : Indicates whether replication traffic from/to this partner has been halted.
    unique_name                      : Indicates whether this partner actively mangles object names to avoid name conflicts during replication.
    subnet_label                     : Label of the subnet used to replicate to this partner.
    subnet_type                      : Type of the subnet used to replicate to this partner.
    throttles                        : Throttles used while replicating from/to this partner.
    throttled_bandwidth              : Current bandwidth throttle for this partner, expressed either as megabits per second or as the largest possible 64-bit signed integer
                                       (9223372036854775807) to indicate that there is no throttle. This attribute is superseded by throttled_bandwidth_current.
    throttled_bandwidth_current      : Current bandwidth throttle for this partner, expressed either as megabits per second or as -1 to indicate that there is no throttle.
    throttled_bandwidth_kbps         : Current bandwidth throttle for this partner, expressed either as kilobits per second or as the largest possible 64-bit signed integer
                                       (9223372036854775807) to indicate that there is no throttle. This attribute is superseded by throttled_bandwidth_current_kbps.
    throttled_bandwidth_current_kbps : Current bandwidth throttle for this partner, expressed either as kilobits per second or as -1 to indicate that there is no throttle.
    subnet_network                   : Subnet used to replicate to this partner.
    subnet_netmask                   : Subnet mask used to replicate to this partner.
    volume_collection_list           : List of volume collections that are replicating from/to this partner.
    volume_collection_list_count     : Count of volume collections that are replicating from/to this partner.
    volume_list                      : List of volumes that are replicating from/to this partner.
    volume_list_count                : Count of volumes that are replicating from/to this partner.
    replication_direction            : Direction of replication configured with this partner.
    """
    def pause(self, **kwargs):
        """Pause replication for the specified partner.

        # Parameters
        id : ID of the partner to pause.
        """

        return self._collection.pause(
            self.id,
            **kwargs
        )
    def resume(self, **kwargs):
        """Resume replication for the specified partner.

        # Parameters
        id : ID of the partner to resume.
        """

        return self._collection.resume(
            self.id,
            **kwargs
        )
    def test(self, **kwargs):
        """Test connectivity to the specified partner.

        # Parameters
        id : ID of the partner to test.
        """

        return self._collection.test(
            self.id,
            **kwargs
        )


class ReplicationPartnerList(Collection):
    resource = ReplicationPartner
    resource_type = "replication_partners"

    def pause(self, id, **kwargs):
        """Pause replication for the specified partner.

        # Parameters
        id : ID of the partner to pause.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'pause',
            id=id,
            **kwargs
        )

    def resume(self, id, **kwargs):
        """Resume replication for the specified partner.

        # Parameters
        id : ID of the partner to resume.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'resume',
            id=id,
            **kwargs
        )

    def test(self, id, **kwargs):
        """Test connectivity to the specified partner.

        # Parameters
        id : ID of the partner to test.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'test',
            id=id,
            **kwargs
        )



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions

# global variables
protection_template_to_delete = []
prot_template_name1 = nimosclientbase.get_unique_string(
    "unittestcase-protection-template1")

'''ProtectionTemplate TestCase tests the
    ProtectionTemplate object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for ProtectionTemplate TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for ProtectionTemplate TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_protection_template()
    nimosclientbase.log_footer(request.function.__name__)


def delete_protection_template():
    for protection_template_id in protection_template_to_delete:
        try:
            resp = nimosclientbase.get_nimos_client(
            ).protection_templates.delete(id=protection_template_id)
            assert resp is not None
            log("Deleted protection template with "
                f"Id '{protection_template_id}'")
        except exceptions.NimOSAPIError as ex:
            log(f"Failed with exception message : {str(ex)}")
            raise ex
    protection_template_to_delete.clear()


def create_protection_template(protection_template_name):
    resp = nimosclientbase.get_nimos_client().protection_templates.create(
        name=protection_template_name, description="created by testcase")
    protection_template_id = resp.attrs.get("id")
    protection_template_to_delete.append(protection_template_id)
    assert resp is not None
    log("Created protection template with name "
        f"'{protection_template_name}' and Id '{protection_template_id}'")
    return resp


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_protection_template_name(setup_teardown_for_each_test):
    # only . and : and - are allowed as esp characte..
    try:
        resp = create_protection_template(prot_template_name1)
        assert resp is not None
        get_resp = nimosclientbase.get_nimos_client(
        ).protection_templates.get(id=resp.attrs.get("id"))
        assert get_resp is not None
        assert get_resp.attrs.get("id") == resp.attrs.get("id")
        # retreive all
        getall_resp = nimosclientbase.get_nimos_client(
        ).protection_templates.list(detail=True)
        assert getall_resp is not None
        # by default 3 protection tempalate are on array
        assert getall_resp.__len__() >= 3
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. template name : {prot_template_name1}")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_invalid_protection_template_name(setup_teardown_for_each_test):
    try:
        resp = create_protection_template(prot_template_name1+"_")
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. template name : {prot_template_name1}")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_invalidlength_protection_template_name(setup_teardown_for_each_test):
    try:
        resp = create_protection_template(
            prot_template_name1+prot_template_name1)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log("Failed as expected. template name : "
                f"{prot_template_name1+prot_template_name1}")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_protection_template_name(setup_teardown_for_each_test):
    try:
        resp = create_protection_template(prot_template_name1)
        assert resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. template name : {prot_template_name1}")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_protection_template_for_vss_using_insufficient_arguments(
        setup_teardown_for_each_test):
    try:
        resp = create_protection_template(prot_template_name1)
        assert resp is not None
        # for vss all 3 fields app_sync, app_id and app_server is needed
        update_resp = nimosclientbase.get_nimos_client(
        ).protection_templates.update(id=resp.attrs.get("id"),
                                      app_sync='vss',
                                      app_id='sql2005'
                                      )
        assert update_resp is not None
    except exceptions.NimOSAPIError as ex:
        # fiji and below throws SM_http_bad_request as exception
        if "SM_protpol_not_specified" in str(ex) or "SM_http_bad_request" in str(ex):
            log(f"Failed as expected. App server not specifed")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex

@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_protection_template_for_vss_using_required_arguments(
        setup_teardown_for_each_test):
    try:
        resp = create_protection_template(prot_template_name1)
        assert resp is not None
        # for vss all 3 fields app_sync, app_id and app_server is needed
        update_resp = nimosclientbase.get_nimos_client(

        ).protection_templates.update(id=resp.attrs.get("id"),
                                      app_sync='vss',
                                      app_id='sql2016',
                                      app_server="appserver.ns.com"
                                      )
        assert update_resp is not None
        # verify
        assert update_resp.attrs.get("app_server") == "appserver.ns.com"
        assert update_resp.attrs.get("app_id") == "sql2016"
        assert update_resp.attrs.get("app_sync") == "vss"
        assert update_resp.attrs.get("id") == resp.attrs.get("id")

    except exceptions.NimOSAPIError as ex:
        if "SM_protpol_not_specified" in str(ex):
            log("Failed as expected.app server not specifed")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_check_sql_combination_value_for_vss(setup_teardown_for_each_test):
    try:
        resp = create_protection_template(prot_template_name1)
        assert resp is not None
        update_resp = nimosclientbase.get_nimos_client(
        ).protection_templates.update(
            id=resp.attrs.get("id"),
            app_sync='vss',
            app_id='sql2016',
            app_server="appserver.ns.com"
        )
        assert update_resp is not None
        update_resp = nimosclientbase.get_nimos_client(
        ).protection_templates.update(id=resp.attrs.get("id"),
                                      app_sync='vss',
                                      app_id='sql2014',
                                      app_server="appserver.ns.com"
                                      )
        assert update_resp is not None
        update_resp = nimosclientbase.get_nimos_client(
        ).protection_templates.update(id=resp.attrs.get("id"),
                                      app_sync='vss',
                                      app_id='sql2012',
                                      app_server="appserver.ns.com"
                                      )
        assert update_resp is not None
        update_resp = nimosclientbase.get_nimos_client(
        ).protection_templates.update(id=resp.attrs.get("id"),
                                      app_sync='vss',
                                      app_id='sql2008',
                                      app_server="appserver.ns.com"
                                      )
        assert update_resp is not None
    # now try with sql 2019 as app_sync
        update_resp = nimosclientbase.get_nimos_client(
        ).protection_templates.update(id=resp.attrs.get("id"),
                                      app_sync='vss',
                                      app_id='sql2019',
                                      app_server="appserver.ns.com"
                                      )
        assert update_resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log("Failed as expected.correct sql server not provided")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Subnet(Resource):
    """Search subnets information. Many networking tasks require that objects such as replication partners are either on the same network or have a route to a secondary network.
    Subnets let you create logical addressing for selective routing.

    # Parameters
    id                   : Identifier for the initiator group.
    name                 : Name of subnet configuration.
    network              : Subnet network address.
    netmask              : Subnet netmask address.
    type                 : Subnet type. Options include 'mgmt', 'data', and 'mgmt,data'.
    allow_iscsi          : Subnet type.
    allow_group          : Subnet type.
    discovery_ip         : Subnet network address.
    mtu                  : MTU for specified subnet. Valid MTU's are in the 512-16000 range.
    netzone_type         : Specify Network Affinity Zone type for iSCSI enabled subnets. Valid types are Single, Bisect, and EvenOdd for iSCSI subnets.
    vlan_id              : VLAN ID for specified subnet. Valid ID's are in the 1-4094 range.
    creation_time        : Time when this subnet configuration was created.
    last_modified        : Time when this subnet configuration was last modified.
    failover             : Failover setting of the subnet.
    failover_enable_time : Failover for this subnet will be enabled again at the time specified by failover_enable_time.
    """

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class SubnetList(Collection):
    resource = Subnet
    resource_type = "subnets"

    def create(self, **kwargs):
        raise NimOSAPIOperationUnsupported("create operation not supported")

    def delete(self, **kwargs):
        raise NimOSAPIOperationUnsupported("delete operation not supported")

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan

from nimbleclient import exceptions
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
import tests.nimbleclientbase as nimosclientbase
import pytest

'''initiatorgroups tests the Initiator and
     initiatorgroup object funtionality '''

# global variables
initiator_grp_name1 = ""
initiator_grp_name2 = ""
initiator_name1 = ""
initiatorgrp_to_delete = []
initiator_to_delete = []


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for Initiatorgroups Testcase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for initiatorgroups Testcase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    global initiator_grp_name1
    global initiator_grp_name2
    global initiator_name1

    initiator_grp_name1 = nimosclientbase.get_unique_string("IGrpTC-IG1")
    initiator_grp_name2 = nimosclientbase.get_unique_string("IGrpTC-IG2")
    initiator_name1 = nimosclientbase.get_unique_string("IGrpTC-Initiator1")
    yield setup_teardown_for_each_test
    # teardown operations below
    delete_initiatorgroup()
    nimosclientbase.log_footer(request.function.__name__)


def create_initiatorgroup(initiator_grp_name, **kwargs):
    resp = nimosclientbase.get_nimos_client().initiator_groups.create(
        name=initiator_grp_name, **kwargs)
    id = resp.attrs.get("id")
    initiatorgrp_to_delete.append(id)
    assert resp is not None
    log(f"created IG with name '{initiator_grp_name}' and Id '{id}'")
    return resp


def delete_initiatorgroup():
    for ig_id in initiatorgrp_to_delete:
        nimosclientbase.get_nimos_client().initiator_groups.delete(ig_id)
        log(f"Deleted IG with Id '{ig_id}'")
    initiatorgrp_to_delete.clear()


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_create_and_delete_initiatorgroup(setup_teardown_for_each_test):

    ig_resp = create_initiatorgroup(initiator_grp_name1,
                                    access_protocol="iscsi",
                                    description="created by testcase"
                                    )
    # assert the values
    assert ig_resp is not None
    assert ig_resp.attrs.get("description") == "created by testcase"
    assert ig_resp.attrs.get("id") == ig_resp.attrs.get("id")
    assert ig_resp.attrs.get("access_protocol") == "iscsi"
    assert ig_resp.attrs.get("name") == initiator_grp_name1


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_check_mandatory_params_initiatorGroup(
        setup_teardown_for_each_test):

    try:
        ig_resp = create_initiatorgroup(initiator_grp_name1,
                                        description="created by testcase"
                                        )
        # assert the values
        assert ig_resp is not None
        assert ig_resp.attrs.get("description") == "created by testcase"
        assert ig_resp.attrs.get("name") == initiator_grp_name1
    except exceptions.NimOSAPIError as ex:
        if "SM_missing_arg" in str(ex):
            log("Failed as expected. Missing mandatory param")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_initiatorGroup_with_incorrect_access_protocol(
        setup_teardown_for_each_test):
    try:
        access_protocol = "not a valid param. should be FC or iscsi"
        ig_resp = create_initiatorgroup(initiator_grp_name1,
                                        description="created by testcase",
                                        access_protocol=access_protocol
                                        )
        # assert the values
        assert ig_resp is not None
        assert ig_resp.attrs.get("id") == initiatorgrp_to_delete[0]
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. Invalid param value {access_protocol}")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_duplicate_initiatorgroup(setup_teardown_for_each_test):
    try:
        access_protocol = "iscsi"
        ig_resp = create_initiatorgroup(initiator_grp_name1,
                                        description="created by testcase",
                                        access_protocol=access_protocol
                                        )

        ig_resp = create_initiatorgroup(initiator_grp_name1,
                                        description="created by testcase",
                                        access_protocol=access_protocol
                                        )
        # assert the values
        assert ig_resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_duplicate_initiatorgrp" in str(ex):
            log("Failed as expected. "
                f"IG group already present on array '{initiator_grp_name1}'"
                )

        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_initiatorgroup_with_fc_accesss_protocol(
        setup_teardown_for_each_test):
    try:
        access_protocol = "fc"
        ig_resp = create_initiatorgroup(initiator_grp_name1,
                                        description="created by testcase",
                                        access_protocol=access_protocol
                                        )
        # assert the values
        assert ig_resp is not None
        assert ig_resp.attrs.get("id") == initiatorgrp_to_delete[0]
    except exceptions.NimOSAPIError as ex:
        if "SM_fc_svc_not_available" in str(ex):
            log(f"Failed as expected. FC service not available")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_initiatorgroups(setup_teardown_for_each_test):
    try:
        access_protocol = "iscsi"
        iscsi_initiators = [
            {
                "label": "itor1",
                "ip_address": "1.1.1.1",
                "iqn": "iqn.1992-01.com.example:storage.tape1.sys1.xyz"
            }
        ]
        ig_resp1 = create_initiatorgroup(initiator_grp_name1,
                                         description="created by testcase",
                                         access_protocol=access_protocol,
                                         iscsi_initiators=iscsi_initiators
                                         )
        assert ig_resp1 is not None
        assert ig_resp1.attrs.get("id") == initiatorgrp_to_delete[0]

        ig_resp2 = create_initiatorgroup(initiator_grp_name2,
                                         description="created by testcase",
                                         access_protocol=access_protocol,
                                         iscsi_initiators=iscsi_initiators
                                         )
        # assert the values
        assert ig_resp2 is not None
        assert ig_resp2.attrs.get("id") == initiatorgrp_to_delete[1]
        # get all the IG groups and check their initiator
        igrps = nimosclientbase.get_nimos_client(
        ).initiator_groups.list(detail=True, limit=2)

        for igobj in igrps:
            if (igobj.attrs.get("name") == initiator_grp_name1
                    or igobj.attrs.get("name") == initiator_grp_name2):
                assert igobj.attrs.get("description") is not None
                assert igobj.attrs.get("description") == "created by testcase"
                assert igobj.attrs.get("access_protocol") is not None
                assert igobj.attrs.get("access_protocol") == "iscsi"
                assert igobj.attrs.get("iscsi_initiators") is not None
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_update_initiatorgroup(setup_teardown_for_each_test):
    try:
        iscsi_initiators = [
            {
                "label": "itor1",
                "ip_address": "1.1.1.1",
                "iqn": "iqn.1992-01.com.example:storage.tape1.sys1.xyz"
            }
        ]
        description = "modified by testcase"
        access_protocol = "iscsi"
        ig_resp = create_initiatorgroup(
            initiator_grp_name1,
            description="created by testcase",
            access_protocol=access_protocol,
        )
        # assert the values
        assert ig_resp is not None
        # update the target_subnet
        update_resp = nimosclientbase.get_nimos_client(
        ).initiator_groups.update(id=ig_resp.attrs.get("id"),
                                  description=description,
                                  iscsi_initiators=iscsi_initiators)
        assert update_resp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. Invalid param value")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_initiatorgroup_naming_iscsi(setup_teardown_for_each_test):
    iscsi_initiators1 = [
        {
            "label": "initiator1",
            "ip_address": "10.1.1.1",
            "iqn": "iqn.1998-01.com.nimblestorage:intiator1"
        }
    ]

    iscsi_initiators2 = [
        {
            "label": "initiator2",
            "ip_address": "10.1.1.2",
            "iqn": "iqn.1998-01.com.nimblestorage:intiator2"
        }
    ]

    iscsi_initiators3 = [
        {
            "label": "initiator3",
            "ip_address": "10.1.1.3",
            "iqn": "iqn.1998-01.com.nimblestorage:intiator3"
        }
    ]
    try:
        access_protocol = "iscsi"
        ig_resp = create_initiatorgroup(initiator_grp_name1,
                                        description="created by testcase",
                                        access_protocol=access_protocol,
                                        iscsi_initiators=iscsi_initiators1
                                        )
        # assert the values
        assert ig_resp is not None
        assert ig_resp.attrs.get("name") == initiator_grp_name1
        # create one more initiator
        update_resp = nimosclientbase.get_nimos_client().initiators.create(
            initiator_group_id=ig_resp.attrs.get("id"),
            access_protocol=access_protocol,
            label="initiator2",
            iqn="iqn.1998-01.com.nimblestorage:intiator2"
        )
        # assert the values
        assert update_resp is not None
        ig_resp = create_initiatorgroup(initiator_grp_name2,
                                        description="created by testcase",
                                        access_protocol=access_protocol,
                                        iscsi_initiators=iscsi_initiators3
                                        )

        assert ig_resp is not None
        assert ig_resp.attrs.get("name") == initiator_grp_name2
        count = 0
        # get all the initiators and check
        initiatorlist = nimosclientbase.get_nimos_client(
        ).initiators.list(detail=True)

        for initiatorobj in initiatorlist:
            if (initiatorobj.attrs.get("label") == iscsi_initiators1[0]
                ["label"] or
                    initiatorobj.attrs.get("label") == iscsi_initiators2[0]
                    ["label"] or
                    initiatorobj.attrs.get("label") == iscsi_initiators3[0]
                    ["label"]):
                count += 1
        assert count >= 3
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. Invalid param value")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_get_suggested_lun(setup_teardown_for_each_test):

    try:
        iscsi_initiators1 = [
            {
                "label": "initiator1",
                "ip_address": "10.1.1.1",
                "iqn": "iqn.1998-01.com.nimblestorage:intiator1"
            }
        ]
        access_protocol = "iscsi"
        ig_resp = create_initiatorgroup(initiator_grp_name1,
                                        description="created by testcase",
                                        access_protocol=access_protocol,
                                        iscsi_initiators=iscsi_initiators1
                                        )
        assert ig_resp is not None
        suggest_lunresp = nimosclientbase.get_nimos_client(
        ).initiator_groups.suggest_lun(id=ig_resp.attrs.get("id"))
        assert suggest_lunresp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. Invalid param value")
        elif "SM_not_fc_initiatorgrp" in str(ex):
            log(f"Failed as expected due to invalid array setup. FC array groups are required.")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_validate_lun(setup_teardown_for_each_test):
    try:
        iscsi_initiators1 = [
            {
                "label": "initiator1",
                "ip_address": "10.1.1.1",
                "iqn": "iqn.1998-01.com.nimblestorage:intiator1"
            }
        ]
        access_protocol = "iscsi"
        ig_resp = create_initiatorgroup(initiator_grp_name1,
                                        description="created by testcase",
                                        access_protocol=access_protocol,
                                        iscsi_initiators=iscsi_initiators1
                                        )
        assert ig_resp is not None
        validate_lunresp = nimosclientbase.get_nimos_client(
        ).initiator_groups.validate_lun(id=ig_resp.attrs.get("id"), lun=0)
        assert validate_lunresp is not None
    except exceptions.NimOSAPIError as ex:
        if "SM_invalid_arg_value" in str(ex):
            log(f"Failed as expected. Invalid param value")
        elif "SM_not_fc_initiatorgrp" in str(ex):
            log(f"Failed as expected due to inavlid array setup. FC array groups are required")
        else:
            log(f"Failed with exception message : {str(ex)}")
            raise ex



#
#   © Copyright 2020 Hewlett Packard Enterprise Development LP
#
#   This file was auto-generated by the Python SDK generator; DO NOT EDIT.
#


from ...resource import Resource, Collection
from ...exceptions import NimOSAPIOperationUnsupported


class Token(Resource):
    """Manage user's session information.

    # Parameters
    id            : Object identifier for the session token.
    session_token : Token used for authentication.
    username      : User name for the session.
    password      : Password for the user. A password is required for creating a token.
    app_name      : Application name.
    sdk_name      : SDK name.
    source_ip     : IP address from which the session originates.
    creation_time : Time when this token was created.
    last_modified : Time when this token was last modified.
    expiry_time   : Time when this token will expire.
    server_uuid   : Non mandatory 36 character uuid returned by the server. Currently only the witness REST server returns one.
    """
    def report_user_details(self, **kwargs):
        """Reports the user details for this token.

        # Parameters
        id : ID for the session token.
        """

        return self._collection.report_user_details(
            self.id,
            **kwargs
        )

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")


class TokenList(Collection):
    resource = Token
    resource_type = "tokens"

    def report_user_details(self, id, **kwargs):
        """Reports the user details for this token.

        # Parameters
        id : ID for the session token.
        """

        return self._client.perform_resource_action(
            self.resource_type,
            id,
            'report_user_details',
            id=id,
            **kwargs
        )

    def create(self, **kwargs):
        resp = self._client.create_resource(self.resource_type, **kwargs)
        return self.resource(resp['id'], resp, client=self._client, collection=self)

    def update(self, **kwargs):
        raise NimOSAPIOperationUnsupported("update operation not supported")



# (c) Copyright 2020 Hewlett Packard Enterprise Development LP

# @author alok ranjan
import pytest
import tests.nimbleclientbase as nimosclientbase
from tests.nimbleclientbase import SKIPTEST, log_to_file as log
from nimbleclient import exceptions


# global variables
vol_name1 = nimosclientbase.get_unique_string("volumetc-vol1")
initiator_grp_name1 = nimosclientbase.get_unique_string("igrptc-ig1")

'''ACLTestCase tests the acl object functionality '''


@pytest.fixture(scope='module')
def before_running_all_testcase(request):
    log("**** Starting Tests for ACL TestCase *****\n")

    def after_running_all_testcase():
        log("**** Completed Tests for ACL TestCase *****\n")
    request.addfinalizer(after_running_all_testcase)


@pytest.fixture(scope='function')
def setup_teardown_for_each_test(before_running_all_testcase, request):
    # setup operations before yield is called
    nimosclientbase.log_header(request.function.__name__)
    yield setup_teardown_for_each_test
    # teardown operations below
    nimosclientbase.log_footer(request.function.__name__)


@pytest.mark.skipif(SKIPTEST is True,
                    reason="skipped this test as SKIPTEST variable is true")
def test_create_and_delete_acl(setup_teardown_for_each_test):
    try:
        # first create a volume.
        vol_resp = nimosclientbase.get_nimos_client().volumes.create(
            name=vol_name1, size=10)
        assert vol_resp is not None
        # create an initiator group
        ig_resp = nimosclientbase.get_nimos_client().initiator_groups.create(
            name=initiator_grp_name1,
            description="created by testcase",
            access_protocol="iscsi")
        ig_resp is not None
        # create the acl
        acl_resp = nimosclientbase.get_nimos_client(
        ).access_control_records.create(
            apply_to="both",
            initiator_group_id=ig_resp.attrs.get("id"),
            vol_id=vol_resp.attrs.get("id"))
        assert acl_resp is not None
        # assert that it has been applied
        assert acl_resp.attrs.get("vol_id") == vol_resp.attrs.get("id")
        assert acl_resp.attrs.get("initiator_group_name"
                                  ) == ig_resp.attrs.get("name")
        assert acl_resp.attrs.get("apply_to") == "both"

        # cleanup
        acl_resp = nimosclientbase.get_nimos_client(
        ).access_control_records.delete(id=acl_resp.attrs.get("id"))
        ig_resp = nimosclientbase.get_nimos_client().initiator_groups.delete(
            id=ig_resp.attrs.get("id"))
        assert ig_resp is not None
        vol_resp = nimosclientbase.get_nimos_client().volumes.offline(
            id=vol_resp.attrs.get("id"))
        vol_resp = nimosclientbase.get_nimos_client().volumes.delete(
            id=vol_resp.get("id"))
        assert vol_resp is not None
    except exceptions.NimOSAPIError as ex:
        log(f"Failed with exception message : {str(ex)}")
        raise ex
